/*
 LiquidClient (C)QPL Daniel M. Duley 2001-2002. All rights reserved.
 */


#include "liquidclient.h"
#include <qabstractlayout.h>
#include <qlayout.h>
#include <qtoolbutton.h>
#include <qlabel.h>
#include <qdrawutil.h>
#include <qbitmap.h>
#include <kdrawutil.h>
#include <klocale.h>
#include <qdatetime.h>
#include <qimage.h>
#include "workspace.h"
#include "options.h"

#include "embeddata.h"

using namespace KWinInternal;

bool delayButtonPainting;

static QPixmap* close_pix = 0;
static QPixmap* maximize_pix = 0;
static QPixmap* minimize_pix = 0;
static QPixmap* normalize_pix = 0;
static QPixmap* pin_pix = 0;

static QPixmap* close_pix_down = 0;
static QPixmap* maximize_pix_down = 0;
static QPixmap* minimize_pix_down = 0;
static QPixmap* normalize_pix_down = 0;
static QPixmap* pin_pix_down = 0;

static QPixmap* dis_close_pix = 0;
static QPixmap* dis_maximize_pix = 0;
static QPixmap* dis_minimize_pix = 0;
static QPixmap* dis_normalize_pix = 0;
static QPixmap* dis_pin_pix = 0;

static QPixmap* dis_close_pix_down = 0;
static QPixmap* dis_maximize_pix_down = 0;
static QPixmap* dis_minimize_pix_down = 0;
static QPixmap* dis_normalize_pix_down = 0;
static QPixmap* dis_pin_pix_down = 0;

static QPixmap* frame_pix = 0;
static QPixmap* dis_frame_pix = 0;

static bool pixmaps_created = FALSE;

static void adjustColor(const QColor &baseColor, QImage &srcImg,
                        QImage &destImg)
{
    static unsigned int high = qRgb(133, 133, 133);
    static unsigned int low = qRgb(91, 91, 91);
    static unsigned int dark = qRgb(38, 38, 38);
    static unsigned int shadow = qRgb(193, 193, 193);

    unsigned int highColor = baseColor.dark(110).rgb();
    unsigned int lowColor = baseColor.dark(125).rgb();
    unsigned int shadowColor = baseColor.dark(115).rgb();
    unsigned int darkColor = baseColor.dark(145).rgb();

    if(srcImg.depth() > 8){
        unsigned int i, pixel;
        unsigned int count = srcImg.width()*srcImg.height();
        unsigned int *srcData = (unsigned int *)srcImg.bits();
        unsigned int *destData = (unsigned int *)destImg.bits();
        for(i=0; i < count; ++i){
            pixel = srcData[i];
            if(pixel == high)
                destData[i] = highColor;
            else if(pixel == low)
                destData[i] = lowColor;
            else if(pixel == dark)
                destData[i] = darkColor;
            else if(pixel == shadow)
                destData[i] = shadowColor;
            else
                destData[i] = pixel;
        }
    }
    else{
        int x, y;
        unsigned int pixel;
        unsigned int *cTable = (unsigned int *)srcImg.colorTable();
        unsigned int *destData;
        unsigned char *srcData;
        for(y=0; y < srcImg.height(); ++y){
            srcData = (unsigned char *)srcImg.scanLine(y);
            destData = (unsigned int *)destImg.scanLine(y);
            for(x=0; x < srcImg.width(); ++x){
                pixel = srcData[x];
                pixel = cTable[pixel];
                if(pixel == high)
                    destData[x] = highColor;
                else if(pixel == low)
                    destData[x] = lowColor;
                else if(pixel == dark)
                    destData[x] = darkColor;
                else if(pixel == shadow)
                    destData[x] = shadowColor;
                else
                    destData[x] = pixel;
            }
        }
    }
}


static void create_pixmaps()
{
    if(pixmaps_created)
        return;
    pixmaps_created = true;
    QColor aColor = options->color(Options::TitleBar, true);
    QColor iColor = options->color(Options::TitleBar, false);

    QImage srcImg(12, 12, 32);
    QImage destImg(12, 12, 32);
    if(!srcImg.hasAlphaBuffer())
        srcImg.setAlphaBuffer(true);
    if(!destImg.hasAlphaBuffer())
        destImg.setAlphaBuffer(true);

    srcImg = uic_findImage("close_off.xpm");
    adjustColor(aColor, srcImg, destImg);
    close_pix = new QPixmap;
    close_pix->convertFromImage(destImg);
    adjustColor(iColor, srcImg, destImg);
    dis_close_pix = new QPixmap;
    dis_close_pix->convertFromImage(destImg);

    srcImg = uic_findImage("close_on.xpm");
    adjustColor(aColor, srcImg, destImg);
    close_pix_down = new QPixmap;
    close_pix_down->convertFromImage(destImg);
    adjustColor(iColor, srcImg, destImg);
    dis_close_pix_down = new QPixmap;
    dis_close_pix_down->convertFromImage(destImg);

    srcImg = uic_findImage("minimize_off.xpm");
    adjustColor(aColor, srcImg, destImg);
    minimize_pix = new QPixmap;
    minimize_pix->convertFromImage(destImg);
    adjustColor(iColor, srcImg, destImg);
    dis_minimize_pix = new QPixmap;
    dis_minimize_pix->convertFromImage(destImg);

    srcImg = uic_findImage("minimize_on.xpm");
    adjustColor(aColor, srcImg, destImg);
    minimize_pix_down = new QPixmap;
    minimize_pix_down->convertFromImage(destImg);
    adjustColor(iColor, srcImg, destImg);
    dis_minimize_pix_down = new QPixmap;
    dis_minimize_pix_down->convertFromImage(destImg);

    srcImg = uic_findImage("maximize_off.xpm");
    adjustColor(aColor, srcImg, destImg);
    maximize_pix = new QPixmap;
    maximize_pix->convertFromImage(destImg);
    adjustColor(iColor, srcImg, destImg);
    dis_maximize_pix = new QPixmap;
    dis_maximize_pix->convertFromImage(destImg);

    srcImg = uic_findImage("maximize_on.xpm");
    adjustColor(aColor, srcImg, destImg);
    maximize_pix_down = new QPixmap;
    maximize_pix_down->convertFromImage(destImg);
    adjustColor(iColor, srcImg, destImg);
    dis_maximize_pix_down = new QPixmap;
    dis_maximize_pix_down->convertFromImage(destImg);

    srcImg = uic_findImage("restore_off.xpm");
    adjustColor(aColor, srcImg, destImg);
    normalize_pix = new QPixmap;
    normalize_pix->convertFromImage(destImg);
    adjustColor(iColor, srcImg, destImg);
    dis_normalize_pix = new QPixmap;
    dis_normalize_pix->convertFromImage(destImg);

    srcImg = uic_findImage("restore_on.xpm");
    adjustColor(aColor, srcImg, destImg);
    normalize_pix_down = new QPixmap;
    normalize_pix_down->convertFromImage(destImg);
    adjustColor(iColor, srcImg, destImg);
    dis_normalize_pix_down = new QPixmap;
    dis_normalize_pix_down->convertFromImage(destImg);

    srcImg = uic_findImage("sticky_off.xpm");
    adjustColor(aColor, srcImg, destImg);
    pin_pix = new QPixmap;
    pin_pix->convertFromImage(destImg);
    adjustColor(iColor, srcImg, destImg);
    dis_pin_pix = new QPixmap;
    dis_pin_pix->convertFromImage(destImg);

    srcImg = uic_findImage("sticky_on.xpm");
    adjustColor(aColor, srcImg, destImg);
    pin_pix_down = new QPixmap;
    pin_pix_down->convertFromImage(destImg);
    adjustColor(iColor, srcImg, destImg);
    dis_pin_pix_down = new QPixmap;
    dis_pin_pix_down->convertFromImage(destImg);

    QPainter painter;
    int i;
    frame_pix = new QPixmap(32, 24);
    frame_pix->fill(aColor.rgb());
    painter.begin(frame_pix);
    for(i=0; i < 24; i+=4){
        painter.setPen(aColor.dark(105));
        painter.drawLine(0, i, 32, i);
        painter.setPen(aColor.dark(103));
        painter.drawLine(0, i+1, 32, i+1);
    }
    painter.end();
    dis_frame_pix = new QPixmap(32, 24);
    dis_frame_pix->fill(iColor.rgb());
    painter.begin(dis_frame_pix);
    for(i=0; i < 24; i+=4){
        painter.setPen(iColor.dark(105));
        painter.drawLine(0, i, 32, i);
        painter.setPen(iColor.dark(103));
        painter.drawLine(0, i+1, 32, i+1);
    }
    painter.end();
}

static void delete_pixmaps()
{
    delete close_pix;
    delete maximize_pix;
    delete minimize_pix;
    delete normalize_pix;
    delete pin_pix;

    delete dis_close_pix;
    delete dis_maximize_pix;
    delete dis_minimize_pix;
    delete dis_normalize_pix;
    delete dis_pin_pix;

    delete close_pix_down;
    delete maximize_pix_down;
    delete minimize_pix_down;
    delete normalize_pix_down;
    delete pin_pix_down;

    delete dis_close_pix_down;
    delete dis_maximize_pix_down;
    delete dis_minimize_pix_down;
    delete dis_normalize_pix_down;
    delete dis_pin_pix_down;

    delete frame_pix;
    delete dis_frame_pix;

    pixmaps_created = false;
}

void MyToolButton::paintEvent(QPaintEvent *ev)
{
    if(!bgPix || delayButtonPainting)
        return;

    QPainter p(this);
    p.drawTiledPixmap(0, 0, width(), height(), *bgPix, x(), y());
    if(!offPix)
        return;

    bool on = isOn() || isDown();
    KWinInternal::Client *client = (KWinInternal::Client *)parent();
    if(sticky)
        on = client->isSticky() || on;
    QPixmap *pix = on ? onPix : offPix;
    if(!pix)
        pix = offPix;
    int x = (width()-pix->width())/2;
    int y = (height()-pix->height())/2;
    if(on){
        ++x;
        ++y;
    }
    p.drawPixmap(x, y, *pix);

    if(hasMouse()){
        QColor c = options->color(Options::TitleBar, client->isActive());
        int x2 = width()-1;
        int y2 = height()-1;
        p.setPen(on ? c.dark(120) : c.light(130));
        p.drawLine(0, 0, x2, 0);
        p.drawLine(0, 0, 0, y2);
        p.setPen(on ? c.light(120) : c.dark(130));
        p.drawLine(x2, 0, x2, y2);
        p.drawLine(0, y2, x2, y2);
    }
}


void LiquidClient::slotReset()
{
    *menuTmpPix = miniIcon();
    if(isActive()){
        button[0]->setPixmaps(frame_pix, menuTmpPix, 0);
        button[1]->setPixmaps(frame_pix, pin_pix, pin_pix_down);
        button[2]->setPixmaps(frame_pix, minimize_pix, minimize_pix_down);
        button[3]->setPixmaps(frame_pix, maximize_pix, maximize_pix_down);
        button[4]->setPixmaps(frame_pix, close_pix, close_pix_down);

    }
    else{
        button[0]->setPixmaps(dis_frame_pix, menuTmpPix, 0);
        button[1]->setPixmaps(dis_frame_pix, dis_pin_pix, dis_pin_pix_down);
        button[2]->setPixmaps(dis_frame_pix, dis_minimize_pix, dis_minimize_pix_down);
        button[3]->setPixmaps(dis_frame_pix, dis_maximize_pix, dis_maximize_pix_down);
        button[4]->setPixmaps(dis_frame_pix, dis_close_pix, dis_close_pix_down);
    }
   // if (button[6])
   //     button[6]->setIconSet(isActive() ? *question_mark_pix : *dis_question_mark_pix);
    delayButtonPainting = false;

    setFont(options->font(true));
}


LiquidClient::LiquidClient( Workspace *ws, WId w, QWidget *parent, const char *name )
    : Client(ws, w, parent, name, WResizeNoErase)
{
    delayButtonPainting = false;
    setBackgroundMode(QWidget::NoBackground);
    setFont(options->font(isActive()));
    connect(options, SIGNAL(resetClients()), this, SLOT(slotReset()));

    QGridLayout* g = new QGridLayout(this, 0, 0, 3, 2);
    g->setRowStretch(1, 10);
    g->addWidget(windowWrapper(), 1, 1);
    g->addItem(new QSpacerItem(0, 0, QSizePolicy::Fixed, QSizePolicy::Expanding));

    g->addColSpacing(0, 1);
    g->addColSpacing(2, 1);
    g->addRowSpacing(2, 1);

    button[0] = new MyToolButton(this, 0, i18n("Menu"));
    button[1] = new MyToolButton(this, 0, 0, true);
    button[2] = new MyToolButton(this, 0, i18n("Minimize"));
    button[3] = new ThreeButtonButton(this, 0);
    button[4] = new MyToolButton(this, 0, i18n("Close"));

    button[1]->setTipText(isSticky() ? i18n("Un-Sticky") :
                          i18n("Sticky"));
    button[3]->setTipText(isMaximized() ? i18n("Restore") :
                          i18n("Maximize"));


    QHBoxLayout* hb = new QHBoxLayout;
    g->addLayout(hb, 0, 1);
    hb->addWidget(button[0]);
    int fh = fontMetrics().lineSpacing();

    titlebar = new QSpacerItem(10, fh, QSizePolicy::Expanding,
			       QSizePolicy::Minimum);
    hb->addItem(titlebar);

    hb->addWidget(button[1]);
    hb->addWidget(button[2]);
    hb->addWidget(button[3]);
    hb->addWidget(button[4]);

    for(int i = 0; i < 5; i++) {
        if(!button[i])
	    continue;
	button[i]->setFocusPolicy(NoFocus);
        button[i]->setFixedSize(18, 18);
        button[i]->setAutoRaise(true);
    }
    menuTmpPix = new QPixmap(miniIcon());

    connect(button[0], SIGNAL(pressed()), this, SLOT(menuButtonPressed()));
    connect(button[1], SIGNAL(clicked()), this, (SLOT(toggleSticky())));
    connect(button[2], SIGNAL(clicked()), this, (SLOT(iconify())));
    connect(button[3], SIGNAL(clicked(int)), this, (SLOT(maxButtonClicked(int))));
    connect(button[4], SIGNAL(clicked()), this, (SLOT(closeWindow())));
    if(isActive()){
        button[0]->setPixmaps(frame_pix, menuTmpPix, 0);
        button[1]->setPixmaps(frame_pix, pin_pix, pin_pix_down);
        button[2]->setPixmaps(frame_pix, minimize_pix, minimize_pix_down);
	if(isMaximized())
	  button[3]->setPixmaps(frame_pix, normalize_pix, normalize_pix_down);
	else
	  button[3]->setPixmaps(frame_pix, maximize_pix, maximize_pix_down);
        button[4]->setPixmaps(frame_pix, close_pix, close_pix_down);

    }
    else{
        button[0]->setPixmaps(dis_frame_pix, menuTmpPix, 0);
        button[1]->setPixmaps(dis_frame_pix, dis_pin_pix, dis_pin_pix_down);
        button[2]->setPixmaps(dis_frame_pix, dis_minimize_pix, dis_minimize_pix_down);
	if(isMaximized())
	  button[3]->setPixmaps(dis_frame_pix, dis_normalize_pix, dis_normalize_pix_down);
	else
	  button[3]->setPixmaps(dis_frame_pix, dis_maximize_pix, dis_maximize_pix_down);
        button[4]->setPixmaps(dis_frame_pix, dis_close_pix, dis_close_pix_down);
    }
}

void LiquidClient::activeChange(bool on)
{
    if(on){
        button[0]->setPixmaps(frame_pix, menuTmpPix, 0);
        button[1]->setPixmaps(frame_pix, pin_pix, pin_pix_down);
        button[2]->setPixmaps(frame_pix, minimize_pix, minimize_pix_down);
	if(isMaximized())
	  button[3]->setPixmaps(frame_pix, normalize_pix, normalize_pix_down);
	else
	  button[3]->setPixmaps(frame_pix, maximize_pix, maximize_pix_down);
        button[4]->setPixmaps(frame_pix, close_pix, close_pix_down);

    }
    else{
        button[0]->setPixmaps(dis_frame_pix, menuTmpPix, 0);
        button[1]->setPixmaps(dis_frame_pix, dis_pin_pix, dis_pin_pix_down);
        button[2]->setPixmaps(dis_frame_pix, dis_minimize_pix, dis_minimize_pix_down);
	if(isMaximized())
	  button[3]->setPixmaps(dis_frame_pix, dis_normalize_pix, dis_normalize_pix_down);
	else
	  button[3]->setPixmaps(dis_frame_pix, dis_maximize_pix, dis_maximize_pix_down);
        button[4]->setPixmaps(dis_frame_pix, dis_close_pix, dis_close_pix_down);
    }
    Client::activeChange(on);
}


LiquidClient::~LiquidClient()
{
    delete menuTmpPix;
}


void LiquidClient::resizeEvent( QResizeEvent* e)
{
    Client::resizeEvent(e);
    QRegion rr = rect();
    QRect t = titlebar->geometry();

    if(isVisibleToTLW() && !testWFlags(WStaticContents)){
	QRect cr(2, 2, width()-4, height()-4);
	erase(QRegion(cr).subtract(t));
    }
}

/*!\reimp
 */
void LiquidClient::captionChange( const QString& )
{
    repaint(titlebar->geometry(), false);
}


/*!\reimp
 */
void LiquidClient::maximizeChange(bool m)
{
    if(isActive()){
        if(m)
            button[3]->setPixmaps(frame_pix, normalize_pix, normalize_pix_down);
        else
            button[3]->setPixmaps(frame_pix, maximize_pix, maximize_pix_down);
    }
    else{
        if(m)
            button[3]->setPixmaps(dis_frame_pix, dis_normalize_pix, dis_normalize_pix_down);
        else
            button[3]->setPixmaps(dis_frame_pix, dis_maximize_pix, dis_maximize_pix_down);
    }
    button[3]->setTipText(m ? i18n("Restore") : i18n("Maximize"));
}


/*!\reimp
 */
void LiquidClient::stickyChange(bool s)
{
    button[1]->setTipText(s ? i18n("Un-Sticky") : i18n("Sticky"));
    button[1]->repaint(false);
}

void LiquidClient::paintEvent( QPaintEvent* )
{
    QPainter p(this);
    QRect t = titlebar->geometry();

    int x2 = width()-1;
    int y2 = height()-1;
    p.setPen(options->color(Options::TitleBar, isActive()).light(120));
    p.drawLine(0, 0, x2, 0);
    p.drawLine(0, 0, 0, y2);
    p.setPen(options->color(Options::TitleBar, isActive()).dark(130));
    p.drawLine(0, y2, x2, y2);
    p.drawLine(x2, 0, x2, y2);

    QPixmap *pix = isActive() ? frame_pix : dis_frame_pix;
    p.drawTiledPixmap(1, 1, width()-2, t.height()+3, *pix, 1, 1);
    p.drawTiledPixmap(1, t.height()+4, 2, height()-t.height()-5,
                      *pix, 1, t.height()+4);
    p.drawTiledPixmap(x2-2, t.height()+4, 2, height()-t.height()-5,
                       *pix, 1, t.height()+4);
    p.drawTiledPixmap(3, y2-2, width()-6, 2, *pix, 4, y2-2);

    p.setPen(options->color(Options::TitleBar, isActive()).dark(130));
    p.drawLine(3, t.height()+4, 3, y2-3);
    p.drawLine(3, t.height()+4, x2-3, t.height()+4);
    p.setPen(options->color(Options::TitleBar, isActive()).light(120));
    p.drawLine(x2-3, t.height()+4, x2-3, y2-3);
    p.drawLine(3, y2-3, x2-3, y2-3);

    t.setLeft(t.left()+4);
    t.setRight(t.right()-2);
    p.setFont(options->font(isActive()));
    p.setPen(options->color(Options::TitleBar, isActive()).dark(120));
    int textLen = p.fontMetrics().width(caption());
    if(textLen < t.width()-4){
        p.drawText(t.x()+1, t.y()+1, t.width(), t.height(),
                   AlignCenter|AlignVCenter|SingleLine, caption());
        p.setPen(options->color(Options::Font, isActive()));
        p.drawText(t, AlignCenter|AlignVCenter|SingleLine, caption());
    }
    else{
        p.drawText(t.x()+1, t.y()+1, t.width(), t.height(),
                   AlignLeft|AlignVCenter|SingleLine, caption());
        p.setPen(options->color(Options::Font, isActive()));
        p.drawText(t, AlignLeft|AlignVCenter|SingleLine, caption());
    }

}


void LiquidClient::mouseDoubleClickEvent( QMouseEvent * e )
{
    if(titlebar->geometry().contains(e->pos()))
	workspace()->performWindowOperation(this, options->operationTitlebarDblClick());
}


void LiquidClient::init()
{
    Client::init();
}

void LiquidClient::iconChange()
{
    *menuTmpPix = miniIcon();
    if(isActive())
        button[0]->setPixmaps(frame_pix, menuTmpPix, 0);
    else
        button[0]->setPixmaps(dis_frame_pix, menuTmpPix, 0);
}


/*!
  Indicates that the menu button has been clicked. One press shows
  the window operation menu, a double click closes the window.
 */
void LiquidClient::menuButtonPressed()
{
    static QTime* t = 0;
    static LiquidClient* tc = 0;
    if(!t)
	t = new QTime;

    if(tc != this || t->elapsed() > QApplication::doubleClickInterval()){
        QPoint menupoint(button[0]->rect().bottomLeft().x()-1,
                           button[0]->rect().bottomLeft().y()+2);
        workspace()->clientPopup(this)->popup(
                           button[0]->mapToGlobal(menupoint));

        // Animate the click when the menu button is pressed
        button[0]->animateClick();
    }else
	closeWindow();

    t->start();
    tc = this;
}


void LiquidClient::maxButtonClicked( int button )
{
    switch (button){
    case MidButton:
	maximize(MaximizeVertical);
	break;
    case RightButton:
	maximize(MaximizeHorizontal);
	break;
    default: //LeftButton:
	maximize(MaximizeFull);
	break;
    }
}


LiquidToolClient::LiquidToolClient( Workspace *ws, WId w, QWidget *parent, const char *name )
    : Client( ws, w, parent, name )
{
    connect(options, SIGNAL(resetClients()), this, SLOT(slotReset()));
    setFont(options->font(isActive(), true));

    QGridLayout* g = new QGridLayout(this, 0, 0, 2);
    g->setRowStretch(1, 10);
    g->addWidget(windowWrapper(), 1, 1);
    g->addItem(new QSpacerItem(0, 0, QSizePolicy::Fixed, QSizePolicy::Expanding));

    g->addColSpacing(0, 1);
    g->addColSpacing(2, 1);
    g->addRowSpacing(2, 1);

    closeBtn = new MyToolButton(this, 0);
    connect(closeBtn, SIGNAL(clicked()), this, (SLOT( closeWindow())));
    closeBtn->setFixedSize(14, 14);
    slotReset();

    if(isActive())
        closeBtn->setPixmaps(frame_pix, close_pix, close_pix_down);
    else
        closeBtn->setPixmaps(dis_frame_pix, dis_close_pix, dis_close_pix_down);

    QHBoxLayout* hb = new QHBoxLayout;
    g->addLayout(hb, 0, 1);

    int fh = fontMetrics().lineSpacing()+2;

    titlebar = new QSpacerItem(10, fh, QSizePolicy::Expanding,
			       QSizePolicy::Minimum);
    hb->addItem(titlebar);
    hb->addWidget(closeBtn);
}

LiquidToolClient::~LiquidToolClient()
{
}

void LiquidToolClient::resizeEvent(QResizeEvent* e)
{
    Client::resizeEvent( e );
}

void LiquidToolClient::paintEvent(QPaintEvent*)
{
    QPainter p(this);
    QRect t = titlebar->geometry();
    QRect r = rect();
    qDrawWinPanel(&p, r, colorGroup());
    r.setTop(t.bottom()+1);
    qDrawWinPanel(&p, r, colorGroup());
    p.fillRect(QRect( QPoint(t.topLeft() ), QPoint( width() - t.left(), t.bottom())),
               options->color(Options::TitleBar, isActive()));

    p.setPen(options->color(Options::TitleBar, isActive()).light());
    t.setLeft(t.left() + 4);
    t.setRight(t.right() - 2);
    p.setPen(options->color(Options::Font, isActive()));
    p.setFont(options->font(isActive(), true));
    p.drawText(t, AlignLeft|AlignVCenter|SingleLine, caption());
}


void LiquidToolClient::mouseDoubleClickEvent(QMouseEvent * e)
{
    if(titlebar->geometry().contains(e->pos()))
        workspace()->performWindowOperation(this, options->operationTitlebarDblClick());
    workspace()->requestFocus(this);
}

void LiquidToolClient::init()
{
}

void LiquidToolClient::captionChange(const QString&)
{
    repaint(titlebar->geometry(), FALSE);
}

void LiquidToolClient::activeChange( bool on )
{
    if(on)
        closeBtn->setPixmaps(frame_pix, close_pix, close_pix_down);
    else
        closeBtn->setPixmaps(dis_frame_pix, dis_close_pix, dis_close_pix_down);
    Client::activeChange(on);
}


void LiquidToolClient::slotReset()
{
    if(isActive())
        closeBtn->setPixmaps(frame_pix, close_pix, close_pix_down);
    else
        closeBtn->setPixmaps(dis_frame_pix, dis_close_pix, dis_close_pix_down);
    setFont(options->font(isActive(), true ));
    delayButtonPainting = false;
}

extern "C"
{
  Client * allocate(Workspace * workSpace, WId winId, int tool )
  {
      // (tool)
      //  return new LiquidToolClient(workSpace, winId);
      return new LiquidClient(workSpace, winId);
  }
  void init()
  {
      qInitImages_Liquid();
      create_pixmaps();
  }
  void reset()
  {
      delayButtonPainting = true;
      delete_pixmaps();
      create_pixmaps();
      // Ensure change in tooltip state gets applied
      Workspace::self()->slotResetAllClientsDelayed();
  }
  void deinit()
  {
      delete_pixmaps();
      qCleanupImages_Liquid();
  }
}


#include "liquidclient.moc"
