/*
 LiquidClient (C)QPL Daniel M. Duley 2001-2002. All rights reserved.
 */

#ifndef __LIQUID_CLIENT_H
#define __LIQUID_CLIENT_H
#include <qvariant.h>
#include "client.h"
#include "kwinbutton.h"
class QLabel;
class QSpacerItem;

namespace KWinInternal {

class MyToolButton : public KWinInternal::KWinToolButton
{
public:
    MyToolButton(QWidget *parent=0, const char* name=0, const QString &tip=0,
                 bool isStickyBtn=false)
        : KWinToolButton(parent, name, tip)
    {bgPix = NULL; sticky = isStickyBtn; setBackgroundMode(NoBackground);}
    MyToolButton::~MyToolButton(){;}
    void setPixmaps(QPixmap *bg, QPixmap *off, QPixmap *on)
        {bgPix = bg; onPix = on; offPix = off; repaint(false);}
protected:
    void paintEvent(QPaintEvent *ev);

    bool sticky;
    QPixmap *bgPix, *onPix, *offPix ;
};


class LiquidClient : public KWinInternal::Client
{
    Q_OBJECT
public:
    LiquidClient( Workspace *ws, WId w, QWidget *parent=0, const char *name=0 );
    ~LiquidClient();
protected:
    void resizeEvent( QResizeEvent* );
    void paintEvent( QPaintEvent* );

    void mouseDoubleClickEvent( QMouseEvent * );
    void init();
    void captionChange( const QString& name );
    void iconChange();
    void maximizeChange( bool );
    void stickyChange( bool );
    void activeChange( bool );

private slots:
    void menuButtonPressed();
    void maxButtonClicked( int );
    void slotReset();

private:
    MyToolButton* button[7];
    QSpacerItem* titlebar;
    QPixmap *menuTmpPix; // MyToolButton takes pointers
};

class LiquidToolClient : public Client
{
    Q_OBJECT
public:
    LiquidToolClient( Workspace *ws, WId w, QWidget *parent=0, const char *name=0 );
    ~LiquidToolClient();
protected:
    void resizeEvent( QResizeEvent* );
    void paintEvent( QPaintEvent* );

    void mouseDoubleClickEvent( QMouseEvent * );
    void init();
    void captionChange( const QString& name );
    void activeChange( bool );

private slots:
    void slotReset();
private:
    MyToolButton* closeBtn;
    QSpacerItem* titlebar;
};

/*
  Like QToolButton, but provides a clicked(int) signals that
  has the last pressed mouse button as argument
 */
class ThreeButtonButton: public MyToolButton
{
    Q_OBJECT
public:
  ThreeButtonButton ( QWidget *parent = 0, const char* name = 0, 
		      const QString& tip = 0 )
      : MyToolButton( parent, name, tip )
    {
	connect( this, SIGNAL( clicked() ), this, SLOT( handleClicked() ) );
    }
    ~ThreeButtonButton () {}

signals:
    void clicked( int );

protected:
    void mousePressEvent( QMouseEvent* e )
    {
	last_button = e->button();
	QMouseEvent me ( e->type(), e->pos(), e->globalPos(), LeftButton, e->state() );
	KWinToolButton::mousePressEvent( &me );
    }

    void mouseReleaseEvent( QMouseEvent* e )
    {
	QMouseEvent me ( e->type(), e->pos(), e->globalPos(), LeftButton, e->state() );
	KWinToolButton::mouseReleaseEvent( &me );
    }

private slots:
    void handleClicked()
    {
	emit clicked( last_button );
    }

private:
    int last_button;

};

};

#endif
