/*
 LiquidMack (C)QPL Daniel M. Duley 2001-2002. All rights reserved.
 */


#include "liquidmack.h"
#include <qabstractlayout.h>
#include <qlayout.h>
#include <qtoolbutton.h>
#include <qlabel.h>
#include <qdrawutil.h>
#include <qbitmap.h>
#include <kdrawutil.h>
#include <klocale.h>
#include <qdatetime.h>
#include <qimage.h>
#include "workspace.h"
#include "options.h"

using namespace KWinInternal;

static unsigned char iconify_bits[] = {
     0x00, 0x00, 0x00, 0x18, 0x18, 0x00, 0x00, 0x00};

static unsigned char maximize_bits[] = {
    0x00, 0x7e, 0x7e, 0x66, 0x66, 0x7e, 0x7e, 0x00};

static unsigned char minmax_bits[] = {
    0x0c, 0x18, 0x33, 0x67, 0xcf, 0x9f, 0x3f, 0x3f};

static unsigned char sticky_bits[] = {
    0x00, 0x18, 0x18, 0x7e, 0x7e, 0x18, 0x18, 0x00};

static unsigned char unsticky_bits[] = {
    0x00, 0x00, 0x00, 0x7e, 0x7e, 0x00, 0x00, 0x00};

static QPixmap* frame_pix = 0;
static QPixmap* dis_frame_pix = 0;

static bool pixmaps_created = FALSE;

static void create_pixmaps()
{
    if(pixmaps_created)
        return;
    pixmaps_created = true;
    QColor aColor = options->color(Options::TitleBar, true);
    QColor iColor = options->color(Options::TitleBar, false);

    QPainter painter;
    int i;
    frame_pix = new QPixmap(32, 24);
    frame_pix->fill(aColor.rgb());
    painter.begin(frame_pix);
    for(i=0; i < 24; i+=4){
        painter.setPen(aColor.dark(105));
        painter.drawLine(0, i, 32, i);
        painter.setPen(aColor.dark(103));
        painter.drawLine(0, i+1, 32, i+1);
    }
    painter.end();
    dis_frame_pix = new QPixmap(32, 24);
    dis_frame_pix->fill(iColor.rgb());
    painter.begin(dis_frame_pix);
    for(i=0; i < 24; i+=4){
        painter.setPen(iColor.dark(105));
        painter.drawLine(0, i, 32, i);
        painter.setPen(iColor.dark(103));
        painter.drawLine(0, i+1, 32, i+1);
    }
    painter.end();
}

static void delete_pixmaps()
{
    delete frame_pix;
    delete dis_frame_pix;

    pixmaps_created = false;
}

void MyToolButton::paintEvent(QPaintEvent *ev)
{
    QPainter p(this);

    KWinInternal::Client *client = (KWinInternal::Client *)parent();
    bool on = isOn() || isDown();
    if(sticky)
        on = client->isSticky() || on;
    QColorGroup g = options->colorGroup(Options::TitleBar,
                                        client->isActive());
    int x2 = width()-1;
    int y2 = height()-1;
    // outer frame
    p.setPen(g.mid());
    p.drawLine(0, 0, x2, 0);
    p.drawLine(0, 0, 0, y2);
    p.setPen(g.background().light(110));
    p.drawLine(x2, 0, x2, y2);
    p.drawLine(0, x2, y2, x2);
    p.setPen(g.dark());
    p.drawRect(1, 1, width()-2, height()-2);
    // inner frame
    //g = options->colorGroup(Options::ButtonBg, client->isActive());
    g = colorGroup();
    p.fillRect(3, 3, width()-6, height()-6, g.background());
    p.setPen(isDown() ? g.mid() : g.light());
    p.drawLine(2, 2, x2-2, 2);
    p.drawLine(2, 2, 2, y2-2);
    p.setPen(isDown() ? g.light() : g.mid());
    p.drawLine(x2-2, 2, x2-2, y2-2);
    p.drawLine(2, x2-2, y2-2, x2-2);

    if(!deco.isNull()){
        if(qGray(g.background().rgb()) > 110)
            p.setPen(Qt::black);
        else
            p.setPen(Qt::white);
        int x = (width()-deco.width())/2;
        int y = (height()-deco.height())/2;
        /*if(on){
            ++x;
            ++y;
        }*/
        p.drawPixmap(x, y, deco);
    }
}


void LiquidMackClient::slotReset()
{
    setFont(options->font(true));
}


LiquidMackClient::LiquidMackClient( Workspace *ws, WId w, QWidget *parent, const char *name )
    : Client(ws, w, parent, name, WResizeNoErase)
{
    setBackgroundMode(QWidget::NoBackground);
    setFont(options->font(isActive()));
    connect(options, SIGNAL(resetClients()), this, SLOT(slotReset()));

    QGridLayout* g = new QGridLayout(this, 0, 0, 3, 2);
    g->setRowStretch(1, 10);
    g->addWidget(windowWrapper(), 1, 1);
    g->addItem(new QSpacerItem(0, 0, QSizePolicy::Fixed, QSizePolicy::Expanding));

    g->addColSpacing(0, 1);
    g->addColSpacing(2, 1);
    g->addRowSpacing(2, 1);

    button[0] = new MyToolButton(this, 0, i18n("Close"));
    button[1] = new MyToolButton(this, 0, 0, true);
    button[2] = new MyToolButton(this, 0, i18n("Minimize"));
    button[3] = new ThreeButtonButton(this, 0);

    button[1]->setTipText(isSticky() ? i18n("Un-Sticky") :
                          i18n("Sticky"));
    button[3]->setTipText(isMaximized() ? i18n("Restore") :
                          i18n("Maximize"));

    QHBoxLayout* hb = new QHBoxLayout;
    g->addLayout(hb, 0, 1);
    hb->addWidget(button[0]);
    int fh = fontMetrics().lineSpacing();

    titlebar = new QSpacerItem(10, fh, QSizePolicy::Expanding,
			       QSizePolicy::Minimum);
    hb->addItem(titlebar);

    hb->addWidget(button[1]);
    hb->addWidget(button[2]);
    hb->addWidget(button[3]);

    for(int i = 0; i < 4; i++) {
        if(!button[i])
	    continue;
	button[i]->setFocusPolicy(NoFocus);
	button[i]->setFixedSize(14, 14);
    }

    connect(button[0], SIGNAL(clicked()), this, (SLOT(closeWindow())));
    connect(button[1], SIGNAL(clicked()), this, (SLOT(toggleSticky())));
    connect(button[2], SIGNAL(clicked()), this, (SLOT(iconify())));
    connect(button[3], SIGNAL(clicked(int)), this, (SLOT(maxButtonClicked(int))));

    if(isSticky())
        button[1]->setBitmap(sticky_bits);
    else
        button[1]->setBitmap(unsticky_bits);
    button[2]->setBitmap(iconify_bits);
    if(isMaximized())
        button[3]->setBitmap(minmax_bits);
    else
        button[3]->setBitmap(maximize_bits);
}

void LiquidMackClient::activeChange(bool on)
{
    Client::activeChange(on);
    int i;
    for(i = 0; i < 4; i++)
        button[i]->repaint(false);
}


LiquidMackClient::~LiquidMackClient()
{
}


void LiquidMackClient::resizeEvent( QResizeEvent* e)
{
    Client::resizeEvent(e);
    QRegion rr = rect();
    QRect t = titlebar->geometry();

    if(isVisibleToTLW() && !testWFlags(WStaticContents)){
	QRect cr(2, 2, width()-4, height()-4);
	erase(QRegion(cr).subtract(t));
    }
}

/*!\reimp
 */
void LiquidMackClient::captionChange( const QString& )
{
    repaint(titlebar->geometry(), false);
}


/*!\reimp
 */
void LiquidMackClient::maximizeChange(bool m)
{
    button[3]->setTipText(m ? i18n("Restore") : i18n("Maximize"));
    if(m)
        button[3]->setBitmap(minmax_bits);
    else
        button[3]->setBitmap(maximize_bits);
}


/*!\reimp
 */
void LiquidMackClient::stickyChange(bool s)
{
    button[1]->setTipText(s ? i18n("Un-Sticky") : i18n("Sticky"));
    if(s)
        button[1]->setBitmap(sticky_bits);
    else
        button[1]->setBitmap(unsticky_bits);
}

void LiquidMackClient::paintEvent( QPaintEvent* )
{
    QPainter p(this);
    QRect t = titlebar->geometry();

    int x2 = width()-1;
    int y2 = height()-1;
    p.setPen(options->color(Options::TitleBar, isActive()).light(120));
    p.drawLine(0, 0, x2, 0);
    p.drawLine(0, 0, 0, y2);
    p.setPen(options->color(Options::TitleBar, isActive()).dark(130));
    p.drawLine(0, y2, x2, y2);
    p.drawLine(x2, 0, x2, y2);

    QPixmap *pix = isActive() ? frame_pix : dis_frame_pix;
    p.drawTiledPixmap(1, 1, width()-2, t.height()+3, *pix, 1, 1);
    p.drawTiledPixmap(1, t.height()+4, 2, height()-t.height()-5,
                      *pix, 1, t.height()+4);
    p.drawTiledPixmap(x2-2, t.height()+4, 2, height()-t.height()-5,
                       *pix, 1, t.height()+4);
    p.drawTiledPixmap(3, y2-2, width()-6, 2, *pix, 4, y2-2);

    p.setPen(options->color(Options::TitleBar, isActive()).dark(130));
    p.drawLine(3, t.height()+4, 3, y2-3);
    p.drawLine(3, t.height()+4, x2-3, t.height()+4);
    p.setPen(options->color(Options::TitleBar, isActive()).light(120));
    p.drawLine(x2-3, t.height()+4, x2-3, y2-3);
    p.drawLine(3, y2-3, x2-3, y2-3);

    t.setLeft(t.left()+4);
    t.setRight(t.right()-2);
    p.setFont(options->font(isActive()));
    p.setPen(options->color(Options::TitleBar, isActive()).dark(120));
    int textLen = p.fontMetrics().width(caption());
    QPixmap icon(miniIcon());
    if(icon.isNull()){
        if(textLen < t.width()-4){
            p.drawText(t.x()+1, t.y()+1, t.width(), t.height(),
                       AlignCenter|AlignVCenter|SingleLine, caption());
            p.setPen(options->color(Options::Font, isActive()));
            p.drawText(t, AlignCenter|AlignVCenter|SingleLine, caption());
        }
        else{
            p.drawText(t.x()+1, t.y()+1, t.width(), t.height(),
                       AlignLeft|AlignVCenter|SingleLine, caption());
            p.setPen(options->color(Options::Font, isActive()));
            p.drawText(t, AlignLeft|AlignVCenter|SingleLine, caption());
        }
    }
    else{
        if(textLen < t.width()-18){
            int x = t.x()+((t.width()-(textLen+18))/2);
            p.drawPixmap(x, t.y(), icon);
            x+=18;
            p.drawText(x+1, t.y()+1, t.right()-x, t.height(),
                       AlignLeft|AlignVCenter|SingleLine, caption());
            p.setPen(options->color(Options::Font, isActive()));
            p.drawText(x, t.y(), t.right()-x, t.height(),
                       AlignLeft|AlignVCenter|SingleLine, caption());
        }
        else{
            int x = t.x();
            p.drawPixmap(x, t.y(), icon);
            x += 18;
            p.drawText(x+1, t.y()+1, t.right()-x, t.height(),
                       AlignLeft|AlignVCenter|SingleLine, caption());
            p.setPen(options->color(Options::Font, isActive()));
            p.drawText(x, t.y(), t.right()-x, t.height(),
                       AlignLeft|AlignVCenter|SingleLine, caption());
        }
    }
}


void LiquidMackClient::mouseDoubleClickEvent( QMouseEvent * e )
{
    if(titlebar->geometry().contains(e->pos()))
	workspace()->performWindowOperation(this, options->operationTitlebarDblClick());
}


void LiquidMackClient::init()
{
    Client::init();
}

void LiquidMackClient::iconChange()
{
    ;
}


/*!
  Indicates that the menu button has been clicked. One press shows
  the window operation menu, a double click closes the window.
 */
void LiquidMackClient::menuButtonPressed()
{
    /*
    static QTime* t = 0;
    static LiquidMackClient* tc = 0;
    if(!t)
	t = new QTime;

    if(tc != this || t->elapsed() > QApplication::doubleClickInterval()){
        QPoint menupoint(button[0]->rect().bottomLeft().x()-1,
                           button[0]->rect().bottomLeft().y()+2);
        workspace()->clientPopup(this)->popup(
                           button[0]->mapToGlobal(menupoint));

        // Animate the click when the menu button is pressed
        button[0]->animateClick();
    }else
	closeWindow();

    t->start();
    tc = this; */
}


void LiquidMackClient::maxButtonClicked( int button )
{
    switch (button){
    case MidButton:
	maximize(MaximizeVertical);
	break;
    case RightButton:
	maximize(MaximizeHorizontal);
	break;
    default: //LeftButton:
	maximize(MaximizeFull);
	break;
    }
}


LiquidMackToolClient::LiquidMackToolClient( Workspace *ws, WId w, QWidget *parent, const char *name )
    : Client( ws, w, parent, name )
{
    connect(options, SIGNAL(resetClients()), this, SLOT(slotReset()));
    setFont(options->font(isActive(), true));

    QGridLayout* g = new QGridLayout(this, 0, 0, 2);
    g->setRowStretch(1, 10);
    g->addWidget(windowWrapper(), 1, 1);
    g->addItem(new QSpacerItem(0, 0, QSizePolicy::Fixed, QSizePolicy::Expanding));

    g->addColSpacing(0, 1);
    g->addColSpacing(2, 1);
    g->addRowSpacing(2, 1);

    closeBtn = new MyToolButton(this, 0);
    connect(closeBtn, SIGNAL(clicked()), this, (SLOT( closeWindow())));
    closeBtn->setMinimumSize(14, 14);
    slotReset();

    QHBoxLayout* hb = new QHBoxLayout;
    g->addLayout(hb, 0, 1);

    int fh = fontMetrics().lineSpacing()+2;

    titlebar = new QSpacerItem(10, fh, QSizePolicy::Expanding,
			       QSizePolicy::Minimum);
    hb->addItem(titlebar);
    hb->addWidget(closeBtn);
}

LiquidMackToolClient::~LiquidMackToolClient()
{
}

void LiquidMackToolClient::resizeEvent(QResizeEvent* e)
{
    Client::resizeEvent( e );
}

void LiquidMackToolClient::paintEvent(QPaintEvent*)
{
    QPainter p(this);
    QRect t = titlebar->geometry();
    QRect r = rect();
    qDrawWinPanel(&p, r, colorGroup());
    r.setTop(t.bottom()+1);
    qDrawWinPanel(&p, r, colorGroup());
    p.fillRect(QRect( QPoint(t.topLeft() ), QPoint( width() - t.left(), t.bottom())),
               options->color(Options::TitleBar, isActive()));

    p.setPen(options->color(Options::TitleBar, isActive()).light());
    t.setLeft(t.left() + 4);
    t.setRight(t.right() - 2);
    p.setPen(options->color(Options::Font, isActive()));
    p.setFont(options->font(isActive(), true));
    p.drawText(t, AlignLeft|AlignVCenter|SingleLine, caption());
}


void LiquidMackToolClient::mouseDoubleClickEvent(QMouseEvent * e)
{
    if(titlebar->geometry().contains(e->pos()))
        workspace()->performWindowOperation(this, options->operationTitlebarDblClick());
    workspace()->requestFocus(this);
}

void LiquidMackToolClient::init()
{
}

void LiquidMackToolClient::captionChange(const QString&)
{
    repaint(titlebar->geometry(), FALSE);
}

void LiquidMackToolClient::activeChange( bool on )
{
    Client::activeChange(on);
}


void LiquidMackToolClient::slotReset()
{
    setFont(options->font(isActive(), true ));
}

extern "C"
{
  Client * allocate(Workspace * workSpace, WId winId, int tool )
  {
      // (tool)
      //  return new LiquidMackToolClient(workSpace, winId);
      return new LiquidMackClient(workSpace, winId);
  }
  void init()
  {
      create_pixmaps();
  }
  void reset()
  {
     delete_pixmaps();
     create_pixmaps();
     // Ensure change in tooltip state gets applied
     Workspace::self()->slotResetAllClientsDelayed();
  }
  void deinit()
  {
      delete_pixmaps();
  }
}


#include "liquidmack.moc"
