#include <qdrawutil.h>
#include <qpainter.h>
#include <qpointarray.h>
#include <qstyleplugin.h>

#include <qcombobox.h>
#include <qmenubar.h>
#include <qpushbutton.h>
#include <qscrollbar.h>
#include <qslider.h>
#include <qtabbar.h>
#include <qtoolbutton.h>
#include <qtoolbar.h>
#include <qpopupmenu.h>
#include <qprogressbar.h>
#include <qwidgetlist.h>
#include <qapplication.h>
#include <qfile.h>
#include <qfileinfo.h>
#include <qdir.h>
#include <qsettings.h>
#include <qobjectlist.h>

#include <kpixmapeffect.h>
#include <kimageeffect.h>

#include "liquid.h"
#include "bitmaps.h"

#include "embeddata.h"
//#include "htmlmasks.h"

#define QCOORDARRLEN(x) sizeof(x)/(sizeof(QCOORD)*2)

static const int windowsItemFrame		=  2; // menu item frame width
static const int windowsSepHeight		=  2; // separator item height
static const int windowsItemHMargin		=  3; // menu item hor text margin
static const int windowsItemVMargin		=  2; // menu item ver text margin
static const int windowsArrowHMargin		=  6; // arrow horizontal margin
static const int windowsTabSpacing		= 12; // space between text and tab
static const int windowsCheckMarkHMargin	=  2; // horiz. margins of check mark
static const int windowsRightBorder		= 12; // right border on windows
static const int windowsCheckMarkWidth		= 12; // checkmarks width on windows

class LiquidStylePlugin : public QStylePlugin
{
public:
    LiquidStylePlugin(){;}
    ~LiquidStylePlugin(){;}

    QStringList keys() const {return(QStringList() << "Liquid");}
    QStyle *create(const QString &);
};

QStyle *LiquidStylePlugin::create(const QString &s)
{
    if(s.lower() == "liquid")
        return(new LiquidStyle());
    return(0);
}

Q_EXPORT_PLUGIN( LiquidStylePlugin )

static const int itemFrame       = 1;
static const int itemHMargin     = 3;
static const int itemVMargin     = 1;
static const int arrowHMargin    = 6;
static const int rightBorder     = 12;

void TransMenuHandler::stripePixmap(KPixmap &pix, const QColor &color)
{
    QImage img(pix.convertToImage());
    KImageEffect::fade(img, 0.9, color);
    int x, y;
    int r, g, b;
    for(y=0; y < img.height(); y+=3){
        unsigned int *data = (unsigned int  *) img.scanLine(y);
        for(x=0; x < img.width(); ++x){
            r = qRed(data[x]);
            g = qGreen(data[x]);
            b = qBlue(data[x]);
            if(r-10)
                r-=10;
            if(g-10)
                g-=10;
            if(b-10)
                b-=10;
            data[x] = qRgb(r, g, b);
        }
    }
    pix.convertFromImage(img);
}

TransMenuHandler::TransMenuHandler(QObject *parent)
    : QObject(parent)
{
    pixDict.setAutoDelete(true);
    reloadSettings();
}

void TransMenuHandler::reloadSettings()
{
    pixDict.clear();
    QColorGroup cg(qApp->palette().active());

    QSettings settings;
    type = settings.readNumEntry("/qt/LiquidMenus/Type", TransStippleBg);
    color = QColor(settings.readNumEntry("/qt/LiquidMenus/Color",
                                         (int)cg.button().rgb()));
    fgColor = cg.text();
    if(type == Custom)
        fgColor = QColor(settings.readNumEntry("/qt/LiquidMenus/TextColor",
                                               (int)fgColor.rgb()));
    opacity = settings.readNumEntry("/qt/LiquidMenus/Opacity", 10);
    shadowText = settings.readBoolEntry("/qt/LiquidMenus/ShadowText", true);

    bgStipple = settings.readBoolEntry("/qt/Liquid/BgStipple", true);
    panelStipple = settings.readBoolEntry("/qt/Liquid/PanelBgStipple", true);
    reverseBtnColor = settings.readBoolEntry("/qt/Liquid/ReverseBtnColor",
                                             false);
    panelCustom = settings.readBoolEntry("/qt/Liquid/CustomPanelColor",
                                         false);
    if(panelCustom)
        customPanelColor = QColor(settings.readNumEntry("/qt/Liquid/PanelColor",
                                                        cg.button().rgb()));
    rectangularHTML = settings.readBoolEntry("/qt/Liquid/HTMLRectangular",
                                             true);
}

bool TransMenuHandler::eventFilter(QObject *obj, QEvent *ev)
{
    QPopupMenu *p = (QPopupMenu *)obj;

    if(ev->type() == QEvent::Show){
        if(type == TransStippleBg || type == TransStippleBtn ||
           type == Custom){
            QApplication::syncX();
            KPixmap *pix = new KPixmap;
            if(p->testWFlags(Qt::WType_Popup)){
                QRect r(p->x(), p->y(), p->width(), p->height());
                QRect deskR = QApplication::desktop()->rect();
                if(r.right() > deskR.right() || r.bottom() > deskR.bottom()){
                    r.setBottom(deskR.bottom());
                    r.setRight(deskR.right());
                }
                *pix = QPixmap::grabWindow(qt_xrootwin(), r.x(), r.y(),
                                           r.width(), r.height());
            }
            else{ // tear off menu
                pix->resize(p->width(), p->height());
                pix->fill(Qt::black.rgb());
            }
            if(type == TransStippleBg){
                stripePixmap(*pix, p->colorGroup().background());
            }
            else if(type == TransStippleBtn){
                stripePixmap(*pix, p->colorGroup().button());
            }
            else{
                KPixmapEffect::fade(*pix, (((float)opacity)+80)*0.01, color);
            }
            pixDict.insert(p->winId(), pix);
        }
    }
    else if(ev->type() == QEvent::Hide){
        if(type == TransStippleBg || type == TransStippleBtn ||
           type == Custom){
            pixDict.remove(p->winId());
        }
    }
    return(false);
}

LiquidStyle::LiquidStyle()
    : KStyle(0, ThreeButtonScrollBar)
{
    headerHoverID = -1;
    highlightWidget = NULL;
    btnDict.setAutoDelete(true);
    bevelFillDict.setAutoDelete(true);
    smallBevelFillDict.setAutoDelete(true);

    rMatrix.rotate(90.0);
    btnBorderImg = new QImage(liquid_findImage("buttonfill"));

    bevelFillPix = new QPixmap;
    bevelFillPix->convertFromImage(liquid_findImage("clear_fill_large"));

    smallBevelFillPix = new QPixmap;
    smallBevelFillPix->convertFromImage(liquid_findImage("clear_fill_small"));

    vsbSliderFillPix = menuPix = NULL;
    mnuHandler = new TransMenuHandler(this);

    int i;
    for(i=0; i < BITMAP_ITEMS; ++i){
        pixmaps[i] = NULL;
    }

    sbLeft = new QBitmap(7, 7, sbarrow_left_bits, true);
    sbLeft->setMask(*sbLeft);
    sbRight = new QBitmap(7, 7, sbarrow_right_bits, true);
    sbRight->setMask(*sbRight);
    sbUp = new QBitmap(7, 7, sbarrow_up_bits, true);
    sbUp->setMask(*sbUp);
    sbDown = new QBitmap(7, 7, sbarrow_down_bits, true);
    sbDown->setMask(*sbDown);

    htmlBtnMaskBmp = NULL;
    htmlBtnFrameBmp = NULL;

    oldPaletteLoaded = false;

#ifdef LIQUID_THREADED
    inPulse = false;
    inApp = true;
    pulseWait = new QWaitCondition();
    pulseThread = new LiquidPulseThread(this);
    pulseThread->start();
#endif
    // for qtconfig's preview widget
    /*bgColor = QColor(189, 196, 222);
    bgHoverColor = bgColor.light(105);
    btnColor = QColor(224, 226, 255);
    btnHoverColor = btnColor.light(105);*/

}

LiquidStyle::~LiquidStyle()
{
#ifdef LIQUID_THREADED
    inApp = false;
    inPulse = false;
    highlightWidget = NULL;
    if(pulseThread){
        pulseThread->wait();
        delete pulseThread;
    }
#endif
    btnDict.clear();
    bevelFillDict.clear();
    smallBevelFillDict.clear();

    delete mnuHandler;

    delete sbUp;
    delete sbDown;
    delete sbLeft;
    delete sbRight;

    if(htmlBtnMaskBmp)
        delete htmlBtnMaskBmp;

    if(htmlBtnFrameBmp)
        delete htmlBtnFrameBmp;

    if(btnBorderImg)
        delete btnBorderImg;

    if(bevelFillPix)
        delete bevelFillPix;

    if(smallBevelFillPix)
        delete smallBevelFillPix;

    if(vsbSliderFillPix)
        delete vsbSliderFillPix;

    if(menuPix)
        delete menuPix;

    int i;
    for(i=0; i < BITMAP_ITEMS; ++i){
        if(pixmaps[i])
            delete pixmaps[i];
    }
}

void LiquidStyle::adjustIntensity(QPixmap *pix, int val)
{
    QImage img = pix->convertToImage();
    int i, total;
    unsigned char r, g, b;
    unsigned int *data;
    if(img.depth() > 8){
        total = img.width()*img.height();
        data = (unsigned int *)img.bits();
    }
    else{
        total = img.numColors();
        data = (unsigned int *)img.colorTable();
    }
    if(val >= 0){
        for(i=0; i < total; ++i){
            r = qRed(data[i]);
            g = qGreen(data[i]);
            b = qBlue(data[i]);
            r = r+val <= 255 ? r+val : 255;
            g = g+val <= 255 ? g+val : 255;
            b = b+val <= 255 ? b+val : 255;
            data[i] = qRgba(r, g, b, qAlpha(data[i]));
        }
    }
    else{
        for(i=0; i < total; ++i){
            r = qRed(data[i]);
            g = qGreen(data[i]);
            b = qBlue(data[i]);
            r = r+val > 0 ? r+val : 0;
            g = g+val > 0 ? g+val : 0;
            b = b+val > 0 ? b+val : 0;
            data[i] = qRgba(r, g, b, qAlpha(data[i]));
        }
    }
    pix->convertFromImage(img);
}

QPixmap* LiquidStyle::adjustHSV(QImage &img, const QColor &c,
                                bool blend) const
{
    QImage *tmp = adjustHSVImage(img, c, blend);
    QPixmap *pix = new QPixmap;
    pix->convertFromImage(*tmp);
    delete tmp;
    return(pix);
}

QImage* LiquidStyle::adjustHSVImage(QImage &img, const QColor &c,
                                    bool blend) const
{
    if(img.depth() != 32)
        img = img.convertDepth(32);
    QImage *dest = new QImage(img.width(), img.height(), 32);
    dest->setAlphaBuffer(true);
    unsigned int *data = (unsigned int *)img.bits();
    unsigned int *destData = (unsigned int*)dest->bits();
    int total = img.width()*img.height();
    int current;
    int delta;
    int destR, destG, destB, alpha;
    int srcR = c.red()+20;
    int srcG = c.green()+20;
    int srcB = c.blue()+20;
    float srcPercent, destPercent;
    for(current=0; current < total; ++current){
        alpha = qAlpha(data[current]);
        delta = 255-qRed(data[current]);
        destR = srcR-delta;
        destG = srcG-delta;
        destB = srcB-delta;
        if(destR < 0) destR = 0;
        if(destG < 0) destG = 0;
        if(destB < 0) destB = 0;
        if(destR > 255) destR = 255;
        if(destG > 255) destG = 255;
        if(destB > 255) destB = 255;

        if(blend && alpha != 255 && alpha !=0){
            srcPercent = ((float)alpha)/255.0;
            destPercent = 1.0-srcPercent;
            destR = (int)((srcPercent*destR) + (destPercent*bgColor.red()));
            destG = (int)((srcPercent*destG) + (destPercent*bgColor.green()));
            destB = (int)((srcPercent*destB) + (destPercent*bgColor.blue()));
            alpha = 255;
        }
        destData[current] = qRgba(destR, destG, destB, alpha);
    }
    return(dest);
}


void LiquidStyle::adjustHSV(QPixmap &pix, const QColor &c) const
{
    QImage img = pix.convertToImage();
    pix = *(adjustHSV(img, c, false));
}

QPixmap* LiquidStyle::processEmbedded(const char *label, const QColor &c,
                                      bool blend) const
{
    QImage img(liquid_findImage(label));
    img.detach();
    if(img.isNull()){ // shouldn't happen, been tested
        qWarning("Invalid embedded label %s", label);
        return(NULL);
    }
    return(adjustHSV(img, c, blend));
}

QPixmap* LiquidStyle::getPixmap(BitmapData item) const
{

    if(pixmaps[item])
        return(pixmaps[item]);
    LiquidStyle *ptr = const_cast<LiquidStyle*>(this);
    switch(item){
    case HTMLBtnBorder:
        ptr->pixmaps[HTMLBtnBorder] = processEmbedded("htmlbtnborder", btnColor);
        break;
    case HTMLBtnBorderDown:
        ptr->pixmaps[HTMLBtnBorderDown] = processEmbedded("htmlbtnborder", btnHoverColor);
        break;
    case HTMLCB:
        ptr->pixmaps[HTMLCB] = processEmbedded("htmlcheckbox", bgColor);
        break;
    case HTMLCBDown:
        ptr->pixmaps[HTMLCBDown] = processEmbedded("htmlcheckboxdown", btnColor);
        break;
    case HTMLCBHover:
        ptr->pixmaps[HTMLCBHover] = processEmbedded("htmlcheckbox", btnColor);
        break;
    case HTMLCBDownHover:
        ptr->pixmaps[HTMLCBDownHover] = processEmbedded("htmlcheckboxdown", btnHoverColor);
        break;
    case HTMLRadio:
        ptr->pixmaps[HTMLRadio] = processEmbedded("htmlradio", bgColor);
    case HTMLRadioDown:
        ptr->pixmaps[HTMLRadioDown] = processEmbedded("htmlradiodown", btnColor);
    case HTMLRadioHover:
        ptr->pixmaps[HTMLRadioHover] = processEmbedded("htmlradio", btnColor);
    case HTMLRadioDownHover:
        ptr->pixmaps[HTMLRadioDownHover] = processEmbedded("htmlradiodown", btnHoverColor);
        break;
    case RadioOn:
        ptr->pixmaps[RadioOn] = processEmbedded("radio_down", btnColor, true);
        break;
    case RadioOnHover:
        ptr->pixmaps[RadioOnHover] = processEmbedded("radio_down", btnHoverColor, true);
        break;
    case RadioOffHover:
        ptr->pixmaps[RadioOffHover] = processEmbedded("radio", btnColor, true);
        break;
    case TabDown:
        ptr->pixmaps[TabDown] = processEmbedded("tab", btnColor, true);
        break;
    case TabFocus:
        ptr->pixmaps[TabFocus] = processEmbedded("tab", btnHoverColor, true);
        break;
    case CBDown:
        ptr->pixmaps[CBDown] = processEmbedded("checkboxdown", btnColor, true);
        break;
    case CBDownHover:
        ptr->pixmaps[CBDownHover] = processEmbedded("checkboxdown", btnHoverColor, true);
        break;
    case CBHover:
        ptr->pixmaps[CBHover] = processEmbedded("checkbox", btnColor, true);
        break;
    case HSlider:
        ptr->pixmaps[HSlider] = processEmbedded("sliderarrow", btnColor, true);
        break;
    case VSlider:
        ptr->pixmaps[VSlider] = processEmbedded("sliderarrow", btnColor, true);
        *ptr->pixmaps[VSlider] = ptr->pixmaps[VSlider]->xForm(rMatrix);
        break;
    case RadioOff:
        ptr->pixmaps[RadioOff] = processEmbedded("radio", bgColor, true);
        break;
    case Tab:
        ptr->pixmaps[Tab] = processEmbedded("tab", bgColor, true);
        break;
    case CB:
        ptr->pixmaps[CB] = processEmbedded("checkbox", bgColor, true);
        break;
    case VSBSliderTop:
        ptr->pixmaps[VSBSliderTop] = processEmbedded("sbslider_top", btnColor, true);
        break;
    case VSBSliderBtm:
        ptr->pixmaps[VSBSliderBtm] = processEmbedded("sbslider_btm", btnColor, true);
        break;
    case VSBSliderMid:
        ptr->pixmaps[VSBSliderMid] = processEmbedded("sbslider_mid", btnColor);
        break;
    case VSBSliderTopHover:
        ptr->pixmaps[VSBSliderTopHover] = processEmbedded("sbslider_top", btnHoverColor, true);
        break;
    case VSBSliderBtmHover:
        ptr->pixmaps[VSBSliderBtmHover] = processEmbedded("sbslider_btm", btnHoverColor, true);
        break;
    case VSBSliderMidHover:
        ptr->pixmaps[VSBSliderMidHover] = processEmbedded("sbslider_mid", btnHoverColor);
        break;

    case HSBSliderTop:
        ptr->pixmaps[HSBSliderTop] = processEmbedded("sbslider_top", btnColor, true);
        *ptr->pixmaps[HSBSliderTop] = ptr->pixmaps[HSBSliderTop]->xForm(rMatrix);
        break;
    case HSBSliderBtm:
        ptr->pixmaps[HSBSliderBtm] = processEmbedded("sbslider_btm", btnColor, true);
        *ptr->pixmaps[HSBSliderBtm] = ptr->pixmaps[HSBSliderBtm]->xForm(rMatrix);
        break;
    case HSBSliderMid:
        ptr->pixmaps[HSBSliderMid] = processEmbedded("sbslider_mid", btnColor);
        *ptr->pixmaps[HSBSliderMid] = ptr->pixmaps[HSBSliderMid]->xForm(rMatrix);
        break;
    case HSBSliderTopHover:
        ptr->pixmaps[HSBSliderTopHover] = processEmbedded("sbslider_top", btnHoverColor, true);
        *ptr->pixmaps[HSBSliderTopHover] = ptr->pixmaps[HSBSliderTopHover]->xForm(rMatrix);
        break;
    case HSBSliderBtmHover:
        ptr->pixmaps[HSBSliderBtmHover] = processEmbedded("sbslider_btm", btnHoverColor, true);
        *ptr->pixmaps[HSBSliderBtmHover] = ptr->pixmaps[HSBSliderBtmHover]->xForm(rMatrix);
        break;
    case HSBSliderMidHover:
        ptr->pixmaps[HSBSliderMidHover] = processEmbedded("sbslider_mid", btnHoverColor);
        *ptr->pixmaps[HSBSliderMidHover] = ptr->pixmaps[HSBSliderMidHover]->xForm(rMatrix);
        break;
    case VSBSliderTopBg:
        ptr->pixmaps[VSBSliderTopBg] = processEmbedded("sbgroove_top", bgColor, true);
        break;
    case VSBSliderBtmBg:
        ptr->pixmaps[VSBSliderBtmBg] = processEmbedded("sbgroove_btm", bgColor, true);
        break;
    case VSBSliderMidBg:
        ptr->pixmaps[VSBSliderMidBg] = processEmbedded("sbgroove_mid", bgColor);
        break;
    case HSBSliderTopBg:
        ptr->pixmaps[HSBSliderTopBg] = processEmbedded("sbgroove_top", bgColor, true);
        *ptr->pixmaps[HSBSliderTopBg] = ptr->pixmaps[HSBSliderTopBg]->xForm(rMatrix);
        break;
    case HSBSliderBtmBg:
        ptr->pixmaps[HSBSliderBtmBg] = processEmbedded("sbgroove_btm", bgColor, true);
        *ptr->pixmaps[HSBSliderBtmBg] = ptr->pixmaps[HSBSliderBtmBg]->xForm(rMatrix);
        break;
    case HSBSliderMidBg:
        ptr->pixmaps[HSBSliderMidBg] = processEmbedded("sbgroove_mid", bgColor);
        *ptr->pixmaps[HSBSliderMidBg] = ptr->pixmaps[HSBSliderMidBg]->xForm(rMatrix);
        break;
    default:
        break;
    }
    return(pixmaps[item]);
}


void LiquidStyle::drawClearBevel(QPainter *p, int x, int y, int w, int h,
                                const QColor &c, const QColor &bg) const
{
    LiquidStyle *ptr = const_cast<LiquidStyle*>(this);
    QPen oldPen = p->pen(); // headers need this
    int x2 = x+w-1;
    int y2 = y+h-1;
    // outer dark rect
    p->setPen(c.dark(130));
    p->drawLine(x, y+2, x, y2-2); // l
    p->drawLine(x2, y+2, x2, y2-2); // r
    p->drawLine(x+2, y, x2-2, y); // t
    p->drawLine(x+2, y2, x2-2, y2); // b
    p->drawPoint(x+1, y+1); // tl
    p->drawPoint(x2-1, y+1); // tr
    p->drawPoint(x+1, y2-1); // bl
    p->drawPoint(x2-1, y2-1); // br

    // inner top light lines
    p->setPen(c.light(105));
    p->drawLine(x+2, y+1, x2-2, y+1);
    p->drawLine(x+1, y+2, x2-1, y+2);
    p->drawLine(x+1, y+3, x+2, y+3);
    p->drawLine(x2-2, y+3, x2-1, y+3);
    p->drawPoint(x+1, y+4);
    p->drawPoint(x2-1, y+4);

    // inner bottom light lines
    p->setPen(c.light(110));
    p->drawLine(x+2, y2-1, x2-2, y2-1);
    p->drawLine(x+1, y2-2, x2-1, y2-2);
    p->drawLine(x+1, y2-3, x+2, y2-3);
    p->drawLine(x2-2, y2-3, x2-1, y2-3);
    p->drawPoint(x+1, y2-4);
    p->drawPoint(x2-1, y2-4);

    // inner left mid lines
    //p->setPen(c.light(105));
    p->setPen(c);
    p->drawLine(x+1, y+5, x+1, y2-5);
    p->drawLine(x+2, y+4, x+2, y2-4);

    // inner right mid lines
    p->drawLine(x2-1, y+5, x2-1, y2-5);
    p->drawLine(x2-2, y+4, x2-2, y2-4);

    // fill
    QPixmap *pix;
    if(h >= 32){
        pix = bevelFillDict.find(c.rgb());
        if(!pix){
            pix = new QPixmap(*bevelFillPix);
            adjustHSV(*pix, c);
            ptr->bevelFillDict.insert(c.rgb(), pix);
        }
    }
    else{
        pix = smallBevelFillDict.find(c.rgb());
        if(!pix){
            pix = new QPixmap(*smallBevelFillPix);
            adjustHSV(*pix, c);
            ptr->smallBevelFillDict.insert(c.rgb(), pix);
        }
    }
    p->drawTiledPixmap(x+3, y+3, w-6, h-6, *pix);
    // blend
    int red, green, blue;
    QColor btnColor(c.dark(130));
    red = (btnColor.red() >> 1) + (bg.red() >> 1);
    green = (btnColor.green() >> 1) + (bg.green() >> 1);
    blue = (btnColor.blue() >> 1) + (bg.blue() >> 1);
    btnColor.setRgb(red, green, blue);

    p->setPen(btnColor);
    p->drawPoint(x+1, y);
    p->drawPoint(x, y+1);
    p->drawPoint(x+1, y2);
    p->drawPoint(x, y2-1);

    p->drawPoint(x2-1, y);
    p->drawPoint(x2, y+1);
    p->drawPoint(x2-1, y2);
    p->drawPoint(x2, y2-1);

    p->setPen(oldPen);

}

void LiquidStyle::drawRoundButton(QPainter *painter, const QColor &c,
                                  const QColor &back, int x, int y, int w, int h,
                                  bool supportPushDown, bool pushedDown,
                                  bool autoDefault, bool isMasked) const
{

    if(isMasked && mnuHandler->useRectangularHTML()){
        drawRectangularButton(painter, c, x, y, w, h, pushedDown);
        return;
    }

    if((w < 21 || h < 21) && !autoDefault){
        drawClearBevel(painter, x, y, w, h, c, back);
        return;
    }
    if(supportPushDown){
        --w, --h;
    }
    LiquidStyle *ptr = const_cast<LiquidStyle*>(this);
    QPixmap *pix;
    pix = btnDict.find(c.rgb());
    if(!pix){
        pix = adjustHSV(*btnBorderImg, c, true);
        //pix = new QPixmap(*btnBorderPix);
        //ptr->adjustHSV(*pix, h, s, v);
        ptr->btnDict.insert(c.rgb(), pix);
    }

    int x2 = x+w-1;
    int y2 = y+h-1;
    int bx2 = pix->width()-1;
    int by2 = pix->height()-1;

    QPixmap tmpPix(w, h);
    QPixmap tilePix;
    QPainter p;
    p.begin(&tmpPix);

    // do the fill
    p.drawPixmap(0, 0, *pix, 0, 0, 10, 10); // tl
    p.drawPixmap(x2-9, 0, *pix, bx2-9, 0, 10, 10); // tr
    p.drawPixmap(0, y2-9, *pix, 0, by2-9, 10, 10); // bl
    p.drawPixmap(x2-9, y2-9, *pix, bx2-9, by2-9, 10, 10); // br

    // edges
    tilePix.resize(pix->width()-20, 10);
    // top
    bitBlt(&tilePix, 0, 0, pix, 10, 0, pix->width()-20, 10);
    p.drawTiledPixmap(10, 0, w-20, 10, tilePix);
    // bottom
    bitBlt(&tilePix, 0, 0, pix, 10, by2-9, pix->width()-20, 20);
    p.drawTiledPixmap(10, y2-9, w-20, 10, tilePix);
    // left
    tilePix.resize(10, pix->height()-20);
    bitBlt(&tilePix, 0, 0, pix, 0, 10, 10, pix->height()-20);
    p.drawTiledPixmap(0, 10, 10, h-20, tilePix);
    // right
    bitBlt(&tilePix, 0, 0, pix, bx2-9, 10, 10, pix->height()-20);
    p.drawTiledPixmap(x2-9, 10, 10, h-20, tilePix);

    // middle
    tilePix.resize(pix->width()-20, pix->height()-20);
    bitBlt(&tilePix, 0, 0, pix, 10, 10, pix->width()-20, pix->height()-20);
    p.drawTiledPixmap(10, 10, w-20, h-20, tilePix);

    // do the frame if masked
    if(isMasked){
        if(!htmlBtnFrameBmp){
            ptr->htmlBtnFrameBmp = new QBitmap(37, 26, htmlframe_bits, true);
            ptr->htmlBtnFrameBmp->setMask(*htmlBtnFrameBmp);
        }
        p.setPen(c.dark(180));
        p.drawPixmap(0, 0, *htmlBtnFrameBmp, 0, 0, 10, 10); // tl
        p.drawPixmap(x2-9, 0, *htmlBtnFrameBmp, bx2-9, 0, 10, 10); // tr
        p.drawPixmap(0, y2-9, *htmlBtnFrameBmp, 0, by2-9, 10, 10); // bl
        p.drawPixmap(x2-9, y2-9, *htmlBtnFrameBmp, bx2-9, by2-9, 10, 10); // br
        if(w > 20){
            p.drawLine(10, 0, x2-10, 0);
            p.drawLine(10, y2, x2-10, y2);
        }
        if(h > 20){
            p.drawLine(0, 10, 0, y2-10);
            p.drawLine(x2, 10, x2, y2-10);
        }
    }
    p.end();

    // do the button mask - we don't automask buttons
    QBitmap btnMask(w, h);
    if(isMasked && !htmlBtnMaskBmp){
        ptr->htmlBtnMaskBmp = new QBitmap(37, 26, htmlmask_bits, true);
        ptr->htmlBtnMaskBmp->setMask(*htmlBtnMaskBmp);
    }
    const QBitmap *mask = isMasked ? htmlBtnMaskBmp : pix->mask();
    p.begin(&btnMask);
    p.fillRect(0, 0, w, h, Qt::color0);
    p.drawPixmap(0, 0, *mask, 0, 0, 10, 10); // tl
    p.drawPixmap(x2-9, 0, *mask, bx2-9, 0, 10, 10); // tr
    p.drawPixmap(0, y2-9, *mask, 0, by2-9, 10, 10); // bl
    p.drawPixmap(x2-9, y2-9, *mask, bx2-9, by2-9, 10, 10); // br
    // fills
    p.fillRect(10, 0, w-20, 10, Qt::color1); // top
    p.fillRect(10, y2-9, w-20, 10, Qt::color1); // bottom
    p.fillRect(0, 10, w, h-20, Qt::color1); // middle
    p.end();
    tmpPix.setMask(btnMask);

    if(supportPushDown && pushedDown)
        painter->drawPixmap(x+1, y+1, tmpPix);
    else
        painter->drawPixmap(x, y, tmpPix);
}

void LiquidStyle::drawRectangularButton(QPainter *p, const QColor &c,
                                        int x, int y, int w, int h,
                                        bool sunken, bool hover,
                                        bool isCombo) const
{
    QPen oldPen(p->pen());
    int x2 = x+w-1;
    int y2 = y+h-1;

    QColor c2(isCombo ? hover || sunken ? c :
              qApp->palette().active().background() : c);

    p->setPen(c2.dark(130));
    p->drawRect(x, y, w, h);
    p->setPen(sunken ? c2.dark(120) : c2.light(110));
    p->drawLine(x+1, y+1, x2-1, y+1);
    p->drawLine(x+1, y+1, x+1, y2-1);
    p->setPen(sunken ? c2.light(110) : c2.dark(120));
    p->drawLine(x2-1, y+1, x2-1, y2-1);
    p->drawLine(x+1, y2-1, x2-1, y2-1);
    QPixmap *pix;
    if(h >= 32){
        pix = bevelFillDict.find(c2.rgb());
        if(!pix){
            pix = new QPixmap(*bevelFillPix);
            adjustHSV(*pix, c2);
            const_cast<LiquidStyle*>(this)->
                bevelFillDict.insert(c2.rgb(), pix);
        }
    }
    else{
        pix = smallBevelFillDict.find(c2.rgb());
        if(!pix){
            pix = new QPixmap(*smallBevelFillPix);
            adjustHSV(*pix, c2);
            const_cast<LiquidStyle*>(this)->
                smallBevelFillDict.insert(c2.rgb(), pix);
        }
    }
    p->drawTiledPixmap(x+2, y+2, isCombo? w-18 : w-4, h-4, *pix);

    if(isCombo){
        SFlags flags;
        int ah = h / 3;
        int aw = ah;
        int ax = w - aw - 6;
        int ay = (h - ah) / 2;
        x = x2-16, y+=2;
        y2-=2;
        x2-=2;

        p->setPen(c.dark(130));
        p->drawRect(x, y, 15, h-4);
        p->setPen(sunken ? c.dark(120) : c.light(110));
        p->drawLine(x+1, y+1, x2-1, y+1);
        p->drawLine(x+1, y+1, x+1, y2-1);
        p->setPen(sunken ? c.light(110) : c.dark(120));
        p->drawLine(x2-1, y+1, x2-1, y2-1);
        p->drawLine(x+1, y2-1, x2-1, y2-1);
        pix = smallBevelFillDict.find(c.rgb());
        if(!pix){
            pix = new QPixmap(*smallBevelFillPix);
            adjustHSV(*pix, c);
            const_cast<LiquidStyle*>(this)->
                smallBevelFillDict.insert(c.rgb(), pix);
        }
        p->drawTiledPixmap(x+2, y+2, 11, h-8, *pix);

        flags |= Style_Enabled;
        // Are we "pushed" ?
        if(sunken)
            flags |= Style_Sunken;
        drawPrimitive(PE_ArrowDown, p, QRect(ax, ay, aw, ah),
                      qApp->palette().active(), flags);
    }
    p->setPen(oldPen);
}

// make sure button hover color is different enough from the bg color
QColor LiquidStyle::calcBtnHoverColor(const QColor &btnColor,
                                      const QColor &bgColor)
{
    QColor hoverColor = btnColor.light(110);
    int rd = bgColor.red()-btnColor.red();
    int gd = bgColor.green()-btnColor.green();
    int bd = bgColor.blue()-btnColor.blue();

    if(rd < 0) rd = rd*-1;
    if(gd < 0) gd = gd*-1;
    if(gd < 0) bd = bd*-1;
    if(rd < 20 && bd < 20 && gd < 20){
        int h, s, v;
        hoverColor.hsv(&h, &s, &v);
        if(s < 245)
            s += 10;
        else if(h < 349)
            h += 10;
        else
            h -= 10;
        hoverColor.setHsv(h, s, v);
    }
    return(hoverColor);
}



void LiquidStyle::polish(QWidget *w)
{
    if(w->inherits("KonqIconViewWidget")){
        // reset pixmap, since Konq animation can't handle it
        QPalette pal(w->palette());
        pal.setBrush(QColorGroup::Background, pal.active().background());
        w->setPalette(pal);
        return;
    }
    else if(w->inherits("QMenuBar")){
        QPalette pal(w->palette());
        pal.setBrush(QColorGroup::Button, bgBrush);
        w->setPalette(pal);
        return;
    }
    else if(w->inherits("QTipLabel")){
        w->setPalette(tooltipPalette);
        w->installEventFilter(this);
        return;
    }
    else if(w->inherits("QToolBar")){
        w->setBackgroundMode(QWidget::PaletteBackground);
        return;
    }
    else if(w->inherits("QComboBox") || w->inherits("QPushButton")){
        // Qt messes up metrics with some Xft fonts - force minimum size
        if(!w->isA("KArrowButton") && !w->isA("AppletHandleButton")){
            if(w->minimumHeight() < 21)
                w->setMinimumHeight(21);
        }
        w->installEventFilter(this);
    }
    else if(w->inherits("KToolBarButton") || w->inherits("QRadioButton") ||
            w->inherits("QCheckBox") || w->isA("AppletHandleDrag") ||
            w->inherits("KMiniPagerButton") ||
            w->inherits("TaskContainer")){
        w->installEventFilter(this);
    }
    else if(w->inherits("QLineEdit")){
        QPalette pal = w->palette();
        pal.setBrush(QColorGroup::Base, baseBrush);
        w->setPalette(pal);
        w->installEventFilter(this);
    }
    else if(w->inherits("QScrollBar")){
        w->setBackgroundMode(QWidget::NoBackground);
        w->installEventFilter(this);
        return;
    }
    else if(w->inherits("QHeader")){
        w->setMouseTracking(true);
        w->installEventFilter(this);
    }
    // Rather nasty workaround for the date label in the digital clock.
    // It sets X11ParentRelative but doesn't update properly with
    // pixmapped brushes
    else if(w->inherits("QLabel") && w->parent() &&
       w->parent()->inherits("ClockApplet")){
        w->setBackgroundMode(QWidget::PaletteBackground);
        w->setPalette(qApp->palette());
        return;
    }

    if(w->isTopLevel()){
        if(w->inherits("QPopupMenu")){
            w->setBackgroundMode(QWidget::NoBackground);
            return;
        }
        else if(qstrcmp(qApp->argv()[0], "kicker") == 0){
            qWarning("Got panel toplevel %s", w->className());
            // force extensions and child panels to use proper palette.
            if(w->inherits("Panel")){
                ;
            }
            else{
                // reset palette for everything else
                QPalette pal = qApp->palette();
                pal.setBrush(QColorGroup::Background, bgBrush);
                pal.setColor(QColorGroup::Mid, bgBrush.color().dark(130));
                pal.setColor(QColorGroup::Dark, bgBrush.color().dark(150));
                pal.setColor(QColorGroup::Midlight, bgBrush.color().light(110));
                pal.setColor(QColorGroup::Light, bgBrush.color().light(120));
                w->setPalette(pal);
            }
        }
        return;
    }
    if(qstrcmp(qApp->argv()[0], "kicker") == 0 &&
       w->inherits("KPanelExtension")){ // FIXME - doesn't work
        w->setPalette(qApp->palette());
    }


    bool isViewport = qstrcmp(w->name(), "qt_viewport") == 0 ||
        qstrcmp(w->name(), "qt_clipped_viewport") == 0;
    bool isViewportChild = w->parent() &&
        ((qstrcmp(w->parent()->name(), "qt_viewport") == 0) ||
         (qstrcmp(w->parent()->name(), "qt_clipped_viewport") == 0));

    if(isViewport && w->parent() && qstrcmp(w->parent()->name(), "proxyview") == 0){
        w->setBackgroundMode(QWidget::X11ParentRelative);
        return;
    }
    if(isViewportChild){
        if(w->inherits("QButton") || w->inherits("QComboBox")){
            if(w->parent()){ // heh, only way to test for KHTML children ;-)
                if(w->parent()->parent()){
                    if(w->parent()->parent()->parent() &&
                       w->parent()->parent()->parent()->inherits("KHTMLView")){
                        w->setAutoMask(true);
                        w->setBackgroundMode(QWidget::NoBackground);
                    }
                }
            }
            return;
        }
    }
    if(w->ownPalette() && !w->inherits("QButton") && !w->inherits("QComboBox")){
        return;
    }
    if(w->inherits("PanelButtonBase"))
        return;

    if(w->parent() && w->parent()->isWidgetType() && !((QWidget*)w->parent())->
       palette().active().brush(QColorGroup::Background).pixmap()){
        qWarning("No parent pixmap for child widget %s", w->className());
        return;
    }
    if(!isViewport && !isViewportChild && !w->testWFlags(WType_Popup) &&
       !w->inherits("KDesktop") && !w->inherits("PasswordDlg")){
        if(w->backgroundMode() == QWidget::PaletteBackground ||
           w->backgroundMode() == QWidget::PaletteButton){
            w->setBackgroundMode(QWidget::X11ParentRelative);
        }
    }

    KStyle::polish(w);
}


void LiquidStyle::unPolish(QWidget *w)
{
    if(w->inherits("KonqIconViewWidget") || w->inherits("QMenuBar")){
        w->unsetPalette();
        return;
    }
    if(w->inherits("QTipLabel")){
        w->removeEventFilter(this);
        w->unsetPalette();
    }
    else if(w->inherits("QToolBar") || w->inherits("QPopupMenu") ||
            w->inherits("QMenuBar")){
        w->setBackgroundMode(QWidget::PaletteButton);
    }
    else if(w->inherits("QLabel") && w->parent() &&
            w->parent()->inherits("ClockApplet")){
        w->unsetPalette();
        w->setBackgroundMode(QWidget::X11ParentRelative);
        return;
    }
    else if(w->inherits("KMiniPagerButton") || w->inherits("QLineEdit") ||
            w->inherits("TaskContainer") || w->inherits("QMenuBar")
            || w->inherits("Panel")){
        w->unsetPalette();
    }
    if(w->isTopLevel())
        return;

    // for viewport children, don't just check for NoBackground....
    bool isViewportChild = w->parent() &&
        ((qstrcmp(w->parent()->name(), "qt_viewport") == 0) ||
         (qstrcmp(w->parent()->name(), "qt_clipped_viewport") == 0));

    if(w->backgroundMode() == QWidget::X11ParentRelative || isViewportChild){
        // this ain't right, but better than nothing - I have to go look and
        // see what else uses PaletteButton.
        if(w->inherits("QPushButton"))
            w->setBackgroundMode(QWidget::PaletteButton);
        else
            w->setBackgroundMode(QWidget::PaletteBackground);
    }

    if(isViewportChild)
        w->setAutoMask(false);

    if(w->inherits("QComboBox") || w->inherits("KToolBarButton") ||
       w->inherits("QLineEdit") || w->inherits("QRadioButton") ||
       w->inherits("QCheckBox") || w->inherits("QScrollBar") ||
       w->isA("AppletHandleDrag") || w->inherits("KMiniPagerButton") ||
       w->inherits("TaskContainer") || w->inherits("QPushButton")){
        w->removeEventFilter(this);
    }
    else if(w->inherits("QHeader")){
        w->setMouseTracking(false);
        w->removeEventFilter(this);
    }

    if(w->inherits("QButton") || w->inherits("QComboBox")){
        if(w->parent() && w->parent()->inherits("KHTMLPart")){
            w->setAutoMask(false);
        }
    }

    KStyle::unPolish(w);
}

void LiquidStyle::unPolish(QApplication *app)
{
    if(oldPaletteLoaded)
        app->setPalette(oldPalette, true);
}

void LiquidStyle::polish(QPalette &appPal)
{
    bool loadPalette = false;
    if(!oldPaletteLoaded){
        oldPalette = appPal;
        oldPaletteLoaded = true;
        loadPalette = true;
        QFileInfo fi(QDir::homeDirPath()+"/.qt/qtrc");
        qtrcModificationTime = fi.lastModified();
    }
    else{
        QFileInfo fi(QDir::homeDirPath()+"/.qt/qtrc");
        QDateTime lastModTime = fi.lastModified();
        if(lastModTime <= qtrcModificationTime){
            // color scheme mods should always be in qtrc, not in any of the
            // other config files in Qt's path
            appPal = polishedPalette;
            return;
        }
        else{
            qtrcModificationTime = lastModTime;
            loadPalette = true;
        }

    }

    tooltipPalette = appPal;
    tooltipPalette.setBrush(QColorGroup::Background, QColor(255, 255, 220));
    tooltipPalette.setBrush(QColorGroup::Foreground, Qt::black);

    int i;
    if(loadPalette){
        QSettings settings;
        QString str;
        QStringList strlist;
        strlist = settings.readListEntry("/qt/Palette/active");
        if(strlist.count() == QColorGroup::NColorRoles){
            for(i = 0; i < QColorGroup::NColorRoles; i++)
                appPal.setBrush(QPalette::Active, (QColorGroup::ColorRole) i,
                                QBrush(QColor(strlist[i])));
        }
        strlist = settings.readListEntry("/qt/Palette/inactive");
        if(strlist.count() == QColorGroup::NColorRoles){
            for(i = 0; i < QColorGroup::NColorRoles; i++)
                appPal.setBrush(QPalette::Inactive, (QColorGroup::ColorRole) i,
                                QBrush(QColor(strlist[i])));
        }
        strlist = settings.readListEntry("/qt/Palette/disabled");
        if(strlist.count() == QColorGroup::NColorRoles){
            for(i = 0; i < QColorGroup::NColorRoles; i++)
                appPal.setBrush(QPalette::Disabled, (QColorGroup::ColorRole) i,
                                QBrush(QColor(strlist[i])));
        }
    }

    for(i=0; i < BITMAP_ITEMS; ++i){
        if(pixmaps[i]){
            delete pixmaps[i];
            pixmaps[i] = NULL;
        }
    }
    QWidgetList *list = QApplication::allWidgets();
    QWidgetListIt it( *list );
    QWidget *w;

    btnDict.clear();
    bevelFillDict.clear();
    smallBevelFillDict.clear();



#ifdef LIQUID_THREADED
    if(pulseThread)
        pulseThread->clearCache();
#endif

    // button color stuff
    QColor c(appPal.active().button());
    btnColor = c;
    btnHoverColor = calcBtnHoverColor(c, appPal.active().background());

    // menu/panel pixmap
    if(!menuPix){
        menuPix = new QPixmap;
        menuPix->resize(64, 64);
    }
    c = mnuHandler->usePanelCustomColor() ? mnuHandler->panelCustomColor() :
        appPal.active().button();
    QPainter painter;
    menuPix->fill(c.dark(105).rgb()); // make a little darker
    if(mnuHandler->usePanelStipple()){
        painter.begin(menuPix);
        painter.setPen(c.dark(110));
        for(i=0; i < 63; i+=4){
            painter.drawLine(0, i, 63, i);
            painter.drawLine(0, i+1, 63, i+1);
        }
        painter.end();
    }
    menuBrush.setColor(c);
    menuBrush.setPixmap(*menuPix);

    // pager brush
    c = c.dark(120);
    QPixmap *pix = smallBevelFillDict.find(c.rgb()); // better be NULL ;-)
    if(!pix){
        pix = new QPixmap(*smallBevelFillPix);
        adjustHSV(*pix, c);
        smallBevelFillDict.insert(c.rgb(), pix);
    }
    pagerHoverBrush.setColor(c);
    pagerHoverBrush.setPixmap(*pix);

    c = c.dark(120);
    pix = smallBevelFillDict.find(c.rgb()); // better be NULL ;-)
    if(!pix){
        pix = new QPixmap(*smallBevelFillPix);
        adjustHSV(*pix, c);
        smallBevelFillDict.insert(c.rgb(), pix);
    }
    pagerBrush.setColor(c);
    pagerBrush.setPixmap(*pix);


    // background color stuff
    //c = config->readColorEntry("background", &Qt::lightGray);
    c = appPal.active().background();
    bgColor = c;
    bgHoverColor = bgColor.light(105);
    if(qstrcmp(qApp->argv()[0], "kicker") == 0){
        appPal.setColor(QColorGroup::Mid, menuBrush.color().dark(110));
        appPal.setColor(QColorGroup::Dark, menuBrush.color().dark(130));
        appPal.setColor(QColorGroup::Midlight, menuBrush.color().light(110));
        appPal.setColor(QColorGroup::Light, menuBrush.color().light(115));
        menuBrush.setColor(bgColor);
        appPal.setBrush(QColorGroup::Background, menuBrush);
    }

    // FIXME?
    if(vsbSliderFillPix)
        delete vsbSliderFillPix;
    vsbSliderFillPix = new QPixmap(bevelFillPix->xForm(rMatrix));
    adjustHSV(*vsbSliderFillPix, bgColor);

    // background brush
    QPixmap wallPaper(32, 32);
    wallPaper.fill(c.rgb());
    if(mnuHandler->useBgStipple()){
        painter.begin(&wallPaper);
        for(i=0; i < 32; i+=4){
            painter.setPen(c.dark(105));
            painter.drawLine(0, i, 32, i);
            painter.setPen(c.dark(103));
            painter.drawLine(0, i+1, 32, i+1);
        };
        painter.end();
    }
    bgBrush.setColor(c);
    // we use the pixmap, even if it's solid, because some widgets use
    // the parent pixmap.
    bgBrush.setPixmap(wallPaper);
    if(qstrcmp(qApp->argv()[0], "kicker") != 0 &&
       qstrcmp(qApp->argv()[0], "ksplash") != 0){
        appPal.setBrush(QColorGroup::Background, bgBrush);
    }

    // lineedits, menubars  and taskbar buttons, and other stuff
    //c = config->readColorEntry("windowBackground", &Qt::white);
    c = appPal.active().base();
    QPixmap basePix;
    basePix.resize(32, 32);
    basePix.fill(c.rgb());
    painter.begin(&basePix);
    painter.setPen(c.dark(105));
    for(i=0; i < 32; i+=4){
        painter.drawLine(0, i, 32, i);
        painter.drawLine(0, i+1, 32, i+1);
    };
    painter.end();
    baseBrush.setColor(c);
    baseBrush.setPixmap(basePix);
    it.toFirst();
    QPalette editPalette;
    while ((w=it.current()) != 0 ){
        ++it;
        if(w->inherits("QLineEdit")){
            editPalette = w->palette();
            editPalette.setBrush(QColorGroup::Base, baseBrush);
            w->setPalette(editPalette);
        }
        else if(w->inherits("QMenuBar")){ // force toplevel menubar to behave
            QPalette pal(w->palette());
            pal.setBrush(QColorGroup::Button, bgBrush);
            w->setPalette(pal);
        }
        // palette reset during mouse hover
        else if(w->inherits("TaskContainer"))
            w->setPalette(appPal);
        else if(w->inherits("QLabel") && w->parent() &&
           w->parent()->inherits("ClockApplet")){
            w->setBackgroundMode(QWidget::PaletteBackground);
            w->setPalette(appPal);
        }
        // force Kicker palette
        else if(w->inherits("Panel")){
            w->setPalette(appPal);
        }
        else if(w->inherits("QTipLabel"))
            w->setPalette(tooltipPalette);

    }

    polishedPalette = appPal;
}

void LiquidStyle::drawPrimitive(PrimitiveElement pe, QPainter *p,
                                const QRect &r, const QColorGroup &cg,
                                SFlags flags, const QStyleOption &opt) const
{
    bool down = flags & Style_Down;
    bool on = flags & Style_On;

    switch(pe){
    case PE_TabBarBase:{
        p->setPen(cg.button().dark(158));
        p->drawLine(r.x(), r.y(), r.right()-1, r.y());
        p->drawLine(r.x(), r.y(), r.x(), r.bottom());
        p->drawLine(r.x(), r.bottom(), r.right(), r.bottom());
        p->drawLine(r.right(), r.y()+1, r.right(), r.bottom());
        p->setPen(cg.button().dark(110));
        p->drawLine(r.x()+1, r.y()+1, r.right()-1, r.y()+1);
        p->drawLine(r.x()+1, r.y()+3, r.right()-1, r.y()+3);
        p->setPen(cg.button().light(110));
        p->drawLine(r.x()+1, r.y()+2, r.right()-1, r.y()+2);
        break;
    }
    case PE_ButtonCommand:
    case PE_ButtonDefault:{
        bool sunken = on || down;
        bool hover = flags & Style_MouseOver;
        bool focused = flags & Style_HasFocus;
        QColor newColor;
        if(!mnuHandler->useReverseBtnColor() || cg.button() != btnColor)
            newColor = hover || sunken ? cg.button().light(105) :
                focused ? cg.background() : cg.button();
        else{
            newColor = sunken ? cg.button() : hover ? cg.button().light(105) :
                focused ? cg.button() : cg.background();
        }
        drawRoundButton(p, newColor, cg.background(), r.x(), r.y(),
                        r.width(), r.height(), !widgetAutoMasked, sunken,
                        false, widgetAutoMasked);
        break;
    }
    case PE_ButtonBevel:
    case PE_ButtonDropDown:
    case PE_ButtonTool:{
        bool sunken = on || down;
        bool hover = flags & Style_MouseOver;
        drawClearBevel(p, r.x(), r.y(), r.width(), r.height(),
                       sunken ? btnColor : hover ? btnHoverColor :
                       cg.background(), cg.background());
    }
    case PE_HeaderSection:{
        bool sunken = on || down;
        bool hover = flags & Style_MouseOver;
        if(currentHeader == p->device()){
            int id = currentHeader->sectionAt(r.x());
            hover = id != -1 && id == headerHoverID;
        }
        drawClearBevel(p, r.x(), r.y(), r.width(), r.height(),
                       sunken ? cg.button() : hover ? cg.button().light(105) :
                       cg.background(), cg.background());
        break;
    }
    case PE_FocusRect:{
        p->drawWinFocusRect(r);
        break;
    }
    case PE_ScrollBarSlider:
    case PE_ScrollBarAddPage:
    case PE_ScrollBarSubPage:{
        QPixmap *bgPix = bevelFillDict.find(cg.background().rgb());
        if(!bgPix){
            bgPix = new QPixmap(*bevelFillPix);
            adjustHSV(*bgPix, cg.background());
            const_cast<LiquidStyle*>(this)->bevelFillDict.insert(cg.background().rgb(),
                                                                 bgPix);
        }
        bool isHover = highlightWidget == currentScrollBar;
        if(flags & Style_Horizontal){
            int extent = currentScrollBar->height();
            QRect bgR(extent, 0, currentScrollBar->width()-extent*3+1,
                      extent);
            if(sbBuffer.size() != currentScrollBar->size())
                const_cast<LiquidStyle*>(this)->
                    sbBuffer.resize(currentScrollBar->size());
            QPainter painter;
            painter.begin(&sbBuffer);
            painter.setPen(cg.background().light(105));
            painter.drawLine(bgR.x(), bgR.y()+1, bgR.x()+8, bgR.y()+1);
            painter.setPen(cg.background().dark(110));
            painter.drawLine(bgR.x(), bgR.y()+13, bgR.x()+8, bgR.y()+13);
            painter.drawPixmap(bgR.x(), bgR.y()+2, *bgPix,
                               bgR.x(), bgR.y()+2,
                               8, 11);
            painter.drawPixmap(bgR.x(), bgR.y()+1,
                               *getPixmap(HSBSliderBtmBg));
            painter.drawTiledPixmap(bgR.x()+8, bgR.y()+1, bgR.width()-16,
                                    13, *getPixmap(HSBSliderMidBg));
            painter.setPen(cg.background().light(105));
            painter.drawLine(bgR.right()-8, bgR.y()+1, bgR.right(), bgR.y()+1);
            painter.setPen(cg.background().dark(110));
            painter.drawLine(bgR.right()-8, bgR.y()+13, bgR.right(), bgR.y()+13);
            painter.drawPixmap(bgR.right()-8, bgR.y()+2, *bgPix, 0, 0, 8, 11);
            painter.drawPixmap(bgR.right()-8, bgR.y()+1,
                               *getPixmap(HSBSliderTopBg));
            if(pe == PE_ScrollBarSlider){
                painter.drawPixmap(r.x(), r.y()+1,
                                   isHover ?
                                   *getPixmap(HSBSliderBtmHover) :
                                   *getPixmap(HSBSliderBtm));
                painter.drawTiledPixmap(r.x()+8, r.y()+1, r.width()-16,
                                        13, isHover ?
                                        *getPixmap(HSBSliderMidHover) :
                                        *getPixmap(HSBSliderMid));
                painter.drawPixmap(r.right()-8, r.y()+1, isHover ?
                                   *getPixmap(HSBSliderTopHover) :
                                   *getPixmap(HSBSliderTop));
            }
            //painter.setPen(cg.background().dark(110));
            painter.setPen(cg.background().dark(150));
            painter.drawLine(bgR.x(), bgR.y(), bgR.right(), bgR.y());
            painter.drawLine(bgR.x(), bgR.bottom(), bgR.right(),
                             bgR.bottom());
            painter.end();
        }
        else{
            int extent = currentScrollBar->width();
            QRect bgR(0, extent, extent,
                      currentScrollBar->height()-extent*3+1);

            if(sbBuffer.size() != currentScrollBar->size())
                const_cast<LiquidStyle*>(this)->
                    sbBuffer.resize(currentScrollBar->size());
            QPainter painter;
            painter.begin(&sbBuffer);
            painter.setPen(cg.background().light(105));
            painter.drawLine(bgR.x()+1, bgR.y(), bgR.x()+1, bgR.y()+8);
            painter.setPen(cg.background().dark(110));
            painter.drawLine(bgR.x()+13, bgR.y(), bgR.x()+13, bgR.y()+8);
            painter.drawPixmap(bgR.x()+2, bgR.y(), *bgPix, bgR.x(), bgR.y(),
                               11, 8);
            painter.drawPixmap(bgR.x()+1, bgR.y(),
                               *getPixmap(VSBSliderTopBg));
            painter.drawTiledPixmap(bgR.x()+1, bgR.y()+8, 13,
                                    bgR.height()-16,
                                    *getPixmap(VSBSliderMidBg));

            painter.setPen(cg.background().light(105));
            painter.drawLine(bgR.x()+1, bgR.bottom()-8, bgR.x()+1, bgR.bottom());
            painter.setPen(cg.background().dark(110));
            painter.drawLine(bgR.x()+13, bgR.bottom()-8, bgR.x()+13, bgR.bottom());
            painter.drawPixmap(bgR.x()+2, bgR.bottom()-8, *bgPix,
                               0, 0, 11, 8);
            painter.drawPixmap(bgR.x()+1, bgR.bottom()-8,
                               *getPixmap(VSBSliderBtmBg));
            if(pe == PE_ScrollBarSlider){
                painter.drawPixmap(r.x()+1, r.y(),
                                   isHover ?
                                   *getPixmap(VSBSliderTopHover):
                                   *getPixmap(VSBSliderTop));
                painter.drawTiledPixmap(r.x()+1, r.y()+8, 13,
                                        r.height()-16, isHover ?
                                        *getPixmap(VSBSliderMidHover) :
                                        *getPixmap(VSBSliderMid));
                painter.drawPixmap(r.x()+1, r.bottom()-8,
                                   isHover ? *getPixmap(VSBSliderBtmHover) :
                                   *getPixmap(VSBSliderBtm));
            }
            //painter.setPen(cg.background().dark(110));
            painter.setPen(cg.background().dark(150));
            painter.drawLine(bgR.x(), bgR.y(), bgR.x(), bgR.bottom());
            painter.drawLine(bgR.right(), bgR.y(), bgR.right(),
                             bgR.bottom());
            painter.end();
        }
        p->drawPixmap(r.x(), r.y(), sbBuffer, r.x(), r.y(), r.width(),
                      r.height());

        break;
    }
    case PE_ScrollBarAddLine:{
        QColor c(cg.background());
        QColor pixColor = flags & Style_Down ? cg.button() : c;
        QPixmap *pix = bevelFillDict.find(pixColor.rgb());
        if(!pix){
            pix = new QPixmap(*bevelFillPix);
            adjustHSV(*pix, pixColor);
            const_cast<LiquidStyle*>(this)->
                bevelFillDict.insert(pixColor.rgb(), pix);
        }
        p->setPen(cg.background().dark(150));
        if(flags & Style_Horizontal){
            p->drawLine(r.x(), r.y(), r.right(), r.y());
            p->drawLine(r.x(), r.bottom(), r.right(), r.bottom());
            p->drawLine(r.right(), r.y(), r.right(), r.bottom());
            p->setPen(c.light(105));
            p->drawLine(r.x(), r.y()+1, r.right()-1, r.y()+1);
            p->drawLine(r.x(), r.y()+1, r.x(), r.bottom()-1);
            p->setPen(c.dark(110));
            p->drawLine(r.right()-1, r.y()+1, r.right()-1, r.bottom()-1);
            p->drawLine(r.x(), r.bottom()-1, r.right()-1, r.bottom()-1);
            p->drawPixmap(r.x()+1, r.y()+2, *pix, 8, r.y()+2, r.width()-3,
                          r.height()-4);
        }
        else{
            p->drawLine(r.x(), r.y(), r.x(), r.bottom());
            p->drawLine(r.right(), r.y(), r.right(), r.bottom());
            p->drawLine(r.x(), r.bottom(), r.right(), r.bottom());
            p->setPen(c.light(105));
            p->drawLine(r.x()+1, r.y(), r.right()-1, r.y());
            p->drawLine(r.x()+1, r.y(), r.x()+1, r.bottom()-1);
            p->setPen(c.dark(110));
            p->drawLine(r.right()-1, r.y(), r.right()-1, r.bottom()-1);
            p->drawLine(r.x()+1, r.bottom()-1, r.right()-1, r.bottom()-1);
            p->drawPixmap(r.x()+2, r.y()+1, *pix, r.x()+2, 8,
                          r.width()-4, r.height()-3);
        }
        p->setPen(flags & Style_Down ? Qt::white : Qt::black);
        p->drawPixmap(r.x()+4, r.y()+4, flags & Style_Horizontal ?
                      *sbRight : *sbDown);
        break;
    }
    case PE_ScrollBarSubLine:{
        QColor c(cg.background());
        QColor pixColor = flags & Style_Down ? cg.button() : c;
        QPixmap *pix = bevelFillDict.find(pixColor.rgb());
        if(!pix){
            pix = new QPixmap(*bevelFillPix);
            adjustHSV(*pix, pixColor);
            const_cast<LiquidStyle*>(this)->
                bevelFillDict.insert(pixColor.rgb(), pix);
        }
        bool top = flags & Style_Horizontal ? r.x() == 0 :
            r.y() == 0;
        p->setPen(cg.background().dark(150));
        if(flags & Style_Horizontal){
            if(top){
                p->drawLine(r.x(), r.y(), r.x(), r.bottom());
                p->drawLine(r.x(), r.y(), r.right(), r.y());
                p->drawLine(r.x(), r.bottom(), r.right(), r.bottom());
                p->setPen(c.light(105));
                p->drawLine(r.x()+1, r.y()+1, r.x()+1, r.bottom()-1);
                p->drawLine(r.x()+1, r.y()+1, r.right(), r.y()+1);
                p->setPen(c.dark(110));
                p->drawLine(r.x()+1, r.bottom()-1, r.right(), r.bottom()-1);
                p->drawPixmap(r.x()+2, r.y()+2, *pix, r.x()+2, r.y()+2,
                              r.width()-2, r.height()-4);
            }
            else{
                p->drawLine(r.x(), r.y(), r.right(), r.y());
                p->drawLine(r.x(), r.bottom(), r.right(), r.bottom());
                p->setPen(c.dark(110));
                p->drawLine(r.right(), r.y()+1, r.right(), r.bottom()-1);
                p->setPen(c.light(105));
                p->drawLine(r.x(), r.y()+1, r.right()-1, r.y()+1);
                p->setPen(c.dark(110));
                p->drawLine(r.right()-1, r.y()+1, r.right()-1, r.bottom()-1);
                p->drawLine(r.x(), r.bottom()-1, r.right()-1, r.bottom()-1);
                p->drawPixmap(r.x(), r.y()+2, *pix, 8, r.y()+2, r.width()-2,
                              r.height()-4);
            }

        }
        else{
            if(top){
                p->drawLine(r.x(), r.y(), r.right(), r.y());
                p->drawLine(r.x(), r.y(), r.x(), r.bottom());
                p->drawLine(r.right(), r.y(), r.right(), r.bottom());
                p->setPen(c.light(105));
                p->drawLine(r.x()+1, r.y()+1, r.right()-1, r.y()+1);
                p->drawLine(r.x()+1, r.y()+1, r.x()+1, r.bottom());
                p->setPen(c.dark(110));
                p->drawLine(r.right()-1, r.y()+1, r.right()-1, r.bottom());
                p->drawPixmap(r.x()+2, r.y()+2, *pix,
                              r.x()+2, r.y()+2, r.width()-4, r.height()-2);
            }
            else{
                p->drawLine(r.x(), r.y(), r.x(), r.bottom());
                p->drawLine(r.right(), r.y(), r.right(), r.bottom());
                p->setPen(c.dark(110));
                p->drawLine(r.x()+1, r.bottom(), r.right()-1, r.bottom());
                p->setPen(c.light(105));
                p->drawLine(r.x()+1, r.y(), r.x()+1, r.bottom()-1);
                p->setPen(c.dark(110));
                p->drawLine(r.right()-1, r.y(), r.right()-1, r.bottom()-1);
                p->drawLine(r.x()+1, r.bottom()-1, r.right()-1, r.bottom()-1);
                p->drawPixmap(r.x()+2, r.y(), *pix,
                              r.x()+2, 8, r.width()-4, r.height()-2);
            }
        }
        p->setPen(flags & Style_Down ? Qt::white : Qt::black);
        p->drawPixmap(r.x()+4, r.y()+4, flags & Style_Horizontal ?
                      *sbLeft : *sbUp);
        break;
    }
    case PE_Indicator:{
        bool hover = flags & Style_MouseOver;;
        if(!(flags & Style_Off))
            p->drawPixmap(r.x(), r.y(), hover ?
                          *getPixmap(CBDownHover) : *getPixmap(CBDown));
        else
            p->drawPixmap(r.x(), r.y(), hover ?
                          *getPixmap(CBHover) : *getPixmap(CB));
        break;
    }
    case PE_IndicatorMask:{
        p->setPen(Qt::color1);
        p->drawPixmap(r.x(), r.y(), *getPixmap(CB)->mask());
        break;
    }
    case PE_ExclusiveIndicator:{
        bool hover = flags & Style_MouseOver;;
        if(on || down)
            p->drawPixmap(r.x(), r.y(), hover ?
                          *getPixmap(RadioOnHover) : *getPixmap(RadioOn));
        else
            p->drawPixmap(r.x(), r.y(), hover ?
                          *getPixmap(RadioOffHover) : *getPixmap(RadioOff));

        break;
    }
    case PE_ExclusiveIndicatorMask:{
        p->setPen(Qt::color1);
        p->drawPixmap(r.x(), r.y(), *getPixmap(RadioOn)->mask());
        break;
    }
    case PE_DockWindowResizeHandle:
    case PE_Splitter:{
        drawClearBevel(p, r.x(), r.y(), r.width(), r.height(),
                       highlightWidget == p->device() ?
                       cg.button().light(105) : cg.button(), cg.background());
        break;
    }
    case PE_Panel:{
        bool sunken  = flags & Style_Sunken;
        int lw = opt.isDefault() ? pixelMetric(PM_DefaultFrameWidth)
            : opt.lineWidth();
        if (lw == 2 && sunken){
            QPen oldPen = p->pen();
            int x,y,w,h;
            r.rect(&x, &y, &w, &h);
            int x2 = x+w-1;
            int y2 = y+h-1;
            p->setPen(cg.light());
            p->drawLine(x, y2, x2, y2);
            p->drawLine(x2, y, x2, y2);
            p->setPen(cg.mid());
            p->drawLine(x, y, x2, y);
            p->drawLine(x, y, x, y2);
            p->setPen(cg.midlight());
            p->drawLine(x+1, y2-1, x2-1, y2-1);
            p->drawLine(x2-1, y+1, x2-1, y2-1);
            p->setPen(cg.dark());
            p->drawLine(x+1, y+1, x2-1, y+1);
            p->drawLine(x+1, y+1, x+1, y2-1);
            p->setPen(oldPen);
        }
        else
            KStyle::drawPrimitive(pe, p, r, cg, flags, opt);
        break;
    }
    case PE_PanelTabWidget:{
        p->setPen(cg.background().dark(140));
        p->drawRect(r);
        p->setPen(cg.background());
        p->drawLine(r.right()-1, r.y()+1, r.right()-1, r.bottom()-1);
        p->drawLine(r.x()+1, r.bottom()-1, r.right()-1, r.bottom()-1);
        p->setPen(cg.background().light(130));
        p->drawLine(r.x()+1, r.y()+1, r.right()-1, r.y()+1);
        p->drawLine(r.x()+1, r.y()+1, r.x()+1, r.bottom()-1);
        break;
    }
    case PE_PanelLineEdit:{
        p->setPen(Qt::black);
        p->drawRect(r);
        qDrawShadePanel(p, r.x()+1, r.y()+1, r.width()-2, r.height()-2, cg,
                        true, 1);
        //p->drawRect(r.x()+1, r.y()+1, r.width()-2, r.height()-2);
        break;
    }
    case PE_PanelPopup:{
        int x, y, w ,h;
        r.rect(&x, &y, &w, &h);

        QColor c;
        int t = mnuHandler->transType();
        if(t == None || t == StippledBg || t == TransStippleBg)
            c = cg.background();
        else if(t == StippledBtn || t == TransStippleBtn)
            c = cg.button();
        else
            c = mnuHandler->bgColor();
        p->setPen(c.dark(140));
        p->drawRect(x, y, w, h);
        p->setPen(c.light(120));
        p->drawRect(x+1, y+1, w-2, h-2);
        break;
    }
    case PE_PanelMenuBar:{
        qDrawShadePanel(p, r.x(), r.y(), r.width(), r.height(), cg, false, 1,
                        &bgBrush);
    }
    case PE_PanelDockWindow:{
        // This method draws toolbars, but Qt erases and repaints the
        // background first. Since we don't need a frame or anything,
        // just the background stipple, we set the background mode and just
        // use that. Only problem: KOffice parts don't polish their widgets
        // for some reason, so we need to check if the bgmode is correct.
        // This means it's usually faster, except kindof sucks for KOffice
        // apps ;-)
        if(p->device() && p->device()->devType() == QInternal::Widget){
            // check if background brush is set properly - KOffice parts
            // don't always call polish somehow - may be issue with
            // dlopen parts
            QWidget *w = (QWidget *)p->device();
            if(w->backgroundMode() == Qt::PaletteButton){
                w->setBackgroundMode(Qt::PaletteBackground);
                w->erase();
                QObjectList *children = w->queryList(0, 0, false, false);
                QObjectListIt it(*children);
                QObject *child;
                while((child = it.current()) != 0){
                    ++it;
                    if(child->isWidgetType()){
                        ((QWidget *)child)->erase();
                    }
                }
                delete children;
            }
        }
        break;
    }
    case PE_DockWindowSeparator:{
        if(!(flags & Style_Horizontal)){
            p->setPen(cg.mid());
            p->drawLine(4, r.height()/2, r.width()-5, r.height()/2);
            p->setPen(cg.light());
            p->drawLine(4, r.height()/2+1, r.width()-5, r.height()/2+1);
        }
        else{
            p->setPen(cg.mid());
            p->drawLine(r.width()/2, 4, r.width()/2, r.height()-5);
            p->setPen(cg.light());
            p->drawLine(r.width()/2+1, 4, r.width()/2+1, r.height()-5);
        }
        break;
    }
    case PE_SpinWidgetUp:
    case PE_SpinWidgetDown:{
        bool sunken = on || down;
        bool hover = flags & Style_MouseOver;;
        drawClearBevel(p, r.x(), r.y(), r.width(), r.height(),
                       sunken ? btnColor : hover ? btnHoverColor :
                       cg.background(), cg.background());
        int x = r.x() + (r.width()-7)/2;
        int y = r.y() + (r.height()-7)/2;
        QPen oldPen(p->pen());
        p->setPen(sunken ? Qt::white : cg.buttonText());
        p->drawPixmap(x, y, pe == PE_SpinWidgetUp ? *sbUp : *sbDown);
        p->setPen(oldPen);
        break;
    }

    default:{
        if(pe >= PE_ArrowUp && pe <= PE_ArrowLeft){
            QPen oldPen(p->pen());
            // arrows
            static const QCOORD u_arrow[]={3,1, 4,1, 2,2, 5,2, 1,3, 6,3, 0,4, 7,4, 0,5, 7,5};
            static const QCOORD d_arrow[]={0,2, 7,2, 0,3, 7,3, 1,4, 6,4, 2,5, 5,5, 3,6, 4,6};
            static const QCOORD l_arrow[]={1,3, 1,4, 2,2, 2,5, 3,1, 3,6, 4,0, 4,7, 5,0, 5,7};
            static const QCOORD r_arrow[]={2,0, 2,7, 3,0, 3,7, 4,1, 4,6, 5,2, 5,5, 6,3, 6,4};
            p->setPen(flags & Style_Enabled ? flags & Style_Down  || flags & Style_Sunken ?
                      cg.light() : cg.buttonText() : cg.mid());
            int x = r.x();
            int y = r.y();
            if(r.width() > 8){
                x = x + (r.width()-8)/2;
                y = y + (r.height()-8)/2;
            }
            QPointArray a;
            switch(pe){
            case PE_ArrowUp:
                a.setPoints(QCOORDARRLEN(u_arrow), u_arrow);
                break;
            case PE_ArrowDown:
                a.setPoints(QCOORDARRLEN(d_arrow), d_arrow);
                break;
            case PE_ArrowLeft:
                a.setPoints(QCOORDARRLEN(l_arrow), l_arrow);
                break;
            default:
                a.setPoints(QCOORDARRLEN(r_arrow), r_arrow);
                break;
            }
            a.translate(x, y);
            p->drawLineSegments(a);
            p->setPen(oldPen);
        }
        else
            KStyle::drawPrimitive( pe, p, r, cg, flags, opt);
    }
    }
}

void LiquidStyle::drawKStylePrimitive(KStylePrimitive kpe, QPainter* p,
                                      const QWidget* widget, const QRect &r,
                                      const QColorGroup &cg, SFlags flags,
                                      const QStyleOption &opt) const
{
    switch(kpe){
    case KPE_ToolBarHandle:{
        int x = r.x();
        int y = r.y();
        int x2 = r.x() + r.width()-1;
        int y2 = r.y() + r.height()-1;
        p->setPen(cg.button().dark(120));
        p->drawLine(x+1, y, x2-1, y);
        p->drawLine(x+1, y2, x2-1, y2);
        p->drawLine(x, y+1, x, y2-1);
        p->drawLine(x2, y+1, x2, y2-1);
        p->setPen(cg.background());
        p->drawPoint(x, y);
        p->drawPoint(x2, y);
        p->drawPoint(x, y2);
        p->drawPoint(x2, y2);
        QPixmap *pix = bevelFillDict.find(cg.button().rgb());
        if(!pix){
            pix = new QPixmap(*bevelFillPix);
            adjustHSV(*pix, cg.button());
            const_cast<LiquidStyle*>(this)->
                bevelFillDict.insert(cg.button().rgb(), pix);
        }
        p->drawTiledPixmap(x+1, y+1, r.width()-2, r.height()-2, *pix);
        break;
    }
    case KPE_GeneralHandle:{
        p->fillRect(r.x(), r.y(), r.width(), r.height(),
                    cg.brush(QColorGroup::Background));
        if(widget->inherits("AppletHandleDrag") &&
           mnuHandler->usePanelCustomColor()){
            QColor c(mnuHandler->panelCustomColor());
            drawClearBevel(p, r.x(), r.y(), r.width(), r.height(),
                           highlightWidget == widget ? c.light(110) : c, c);
        }
        else
            drawClearBevel(p, r.x(), r.y(), r.width(), r.height(),
                           highlightWidget == widget ?
                           cg.button().light(105) : cg.button(), cg.button());
        break;
    }
    case KPE_SliderGroove:{
        QColor c = widget->hasFocus() ? cg.background().dark(120) :
            cg.background();
        int x, y, w, h;
        r.rect(&x, &y, &w, &h);
        if(((const QSlider *)widget)->orientation() == Qt::Horizontal){
            int x2 = x+w-1;
            y = y+(h-5)/2;
            p->setPen(c.dark(130));
            p->drawLine(x+1, y, x2-1, y);
            p->setPen(c.dark(150));
            p->drawLine(x, y+1, x2, y+1);
            p->setPen(c.dark(125));
            p->drawLine(x, y+2, x2, y+2);
            p->setPen(c.dark(130));
            p->drawLine(x, y+3, x2, y+3);
            p->setPen(c.dark(120));
            p->drawLine(x, y+4, x2, y+4);
            p->setPen(c.light(110));
            p->drawLine(x+1, y+5, x2-1, y+5);
        }
        else{
            int y2 = y+h-1;
            x = x+(w-5)/2;
            p->setPen(c.dark(130));
            p->drawLine(x, y+1, x, y2-1);
            p->setPen(c.dark(150));
            p->drawLine(x+1, y, x+1, y2);
            p->setPen(c.dark(125));
            p->drawLine(x+2, y, x+2, y2);
            p->setPen(c.dark(130));
            p->drawLine(x+3, y, x+3, y2);
            p->setPen(c.dark(120));
            p->drawLine(x+4, y, x+4, y2);
            p->setPen(c.light(110));
            p->drawLine(x+5, y+1, x+5, y2-1);
        }
        break;
    }
    case KPE_SliderHandle:{
        p->drawPixmap(r.x(), r.y(),
                      ((const QSlider *)widget)->orientation() ==
                      Qt::Horizontal ? *getPixmap(HSlider) :
                      *getPixmap(VSlider));
        break;
    }
    case KPE_ListViewExpander:{
        int radius = (r.width() - 4) / 2;
        int centerx = r.x() + r.width()/2;
        int centery = r.y() + r.height()/2;

        int red, green, blue;
        red = (cg.dark().red() >> 1) + (cg.base().red() >> 1);
        green = (cg.dark().green() >> 1) + (cg.base().green() >> 1);
        blue = (cg.dark().blue() >> 1) + (cg.base().blue() >> 1);
        QColor blendColor(red, green, blue);

        p->setPen(cg.dark());
        p->drawLine(r.x()+1, r.y(), r.right()-1, r.y());
        p->drawLine(r.x()+1, r.bottom(), r.right()-1, r.bottom());
        p->drawLine(r.x(), r.y()+1, r.x(), r.bottom()-1);
        p->drawLine(r.right(), r.y()+1, r.right(), r.bottom()-1);
        p->setPen(blendColor);
        p->drawPoint(r.x(), r.y());
        p->drawPoint(r.right(), r.y());
        p->drawPoint(r.x(), r.bottom());
        p->drawPoint(r.right(), r.bottom());
        p->setPen(cg.light());
        p->drawRect(r.x()+1, r.y()+1, r.width()-2, r.height()-2);
        p->fillRect(r.x()+2, r.y()+2, r.width()-4, r.height()-4,
                    cg.background());
        p->setPen(cg.text());
        p->drawLine(centerx - radius, centery, centerx + radius, centery );
        if(flags & Style_On)
            p->drawLine(centerx, centery - radius, centerx, centery + radius);
        break;
    }
    default:
        KStyle::drawKStylePrimitive(kpe, p, widget, r, cg, flags, opt);
    }
}

void LiquidStyle::drawControl(ControlElement element, QPainter *p,
                              const QWidget *widget, const QRect &r,
                              const QColorGroup &cg, SFlags flags,
                              const QStyleOption &opt) const
{                                                          
    switch (element){
    case CE_ProgressBarGroove:{
        p->setPen(cg.background().dark(150));
        p->drawRect(r);
        p->fillRect(r.x()+1, r.y()+1, r.width()-2, r.height()-2,
                    cg.base());
        break;
    }
    case CE_ProgressBarContents:{
        const QProgressBar *progress = (const QProgressBar *)widget;
        QRect contentsR(subRect(SR_ProgressBarContents, widget));
        double val = progress->progress();
        val = val/progress->totalSteps();
        if(val > 0.0){
            int w = QMIN(contentsR.width(), (int)(val*contentsR.width()));
            if(w > 1){
                int x = contentsR.x();
                int y = contentsR.y();
                int x2 = contentsR.x()+w-1;
                int y2 = contentsR.bottom();
                p->setPen(cg.button().dark(125));
                p->drawLine(x, y, x2, y);
                p->drawLine(x, y, x, y2);
                p->setPen(cg.button().dark(110));
                p->drawLine(x2, y, x2, y2);
                p->drawLine(x, y2, x2, y2);
                if(w > 2){
                    QPixmap *pix = smallBevelFillDict.find(cg.button().rgb());
                    if(!pix){
                        pix = new QPixmap(*smallBevelFillPix);
                        adjustHSV(*pix, cg.button());
                        const_cast<LiquidStyle*>(this)->
                            smallBevelFillDict.insert(cg.button().rgb(), pix);
                    }
                    p->drawTiledPixmap(x+1, y+1,
                                       w-2, contentsR.height()-2, *pix);
                }
            }
        }
        break;
    }
    case CE_ProgressBarLabel:{
        QRect contentsR(subRect(SR_ProgressBarContents, widget));
        if(!contentsR.isValid())
            return;
        QFont font(p->font());
        font.setBold(true);
        p->setFont(font);
        p->setPen(widget->colorGroup().buttonText());
        p->drawText(contentsR, AlignCenter,
                    ((const QProgressBar *)widget)->progressString());
        break;
    }
    case CE_TabBarTab:{
        if(!widget || !widget->parentWidget())
            break;
        const QTabBar *tabBar = (const QTabBar *)widget;
        bool selected = flags & Style_Selected;
        if(tabBar->shape() != QTabBar::RoundedAbove){
            KStyle::drawControl(element, p, widget, r, cg, flags, opt);
            return;
        }
        QPixmap *pix = selected ? getPixmap(TabDown) : getPixmap(Tab);
        QPixmap tilePix;
        p->drawPixmap(r.x(), r.y(), *pix, 0, 0, 9, r.height());
        p->drawPixmap(r.right()-9, r.y(), *pix, pix->width()-9, 0, 9,
                      r.height());
        tilePix.resize(pix->width()-18, r.height());
        bitBlt(&tilePix, 0, 0, pix, 9, 0, pix->width()-18, r.height());
        p->drawTiledPixmap(r.x()+9, r.y(), r.width()-18, r.height(),
                           tilePix);
        QColor c = tabBar->colorGroup().button();
        if(!selected){
            p->setPen(c.dark(158));
            p->drawLine(r.x(), r.bottom(), r.right(), r.bottom());
        }
        break;
    }
    case CE_TabBarLabel:{
        if(opt.isDefault())
            return;
        QTab *t = opt.tab();
        if((flags & Style_HasFocus) && !t->text().isEmpty()){
            const QColor tmp(cg.button().dark(130));
            drawItem(p, QRect(r.x()+1, r.y()+1, r.width(), r.height()),
                     AlignCenter | ShowPrefix, cg,
                     flags & Style_Enabled, 0, t->text(), -1, &tmp);
            drawItem(p, QRect(r.x()+2, r.y()+2, r.width(), r.height()),
                     AlignCenter | ShowPrefix, cg,
                     flags & Style_Enabled, 0, t->text(), -1, &tmp);
        }
        drawItem(p, r, AlignCenter | ShowPrefix, cg, flags & Style_Enabled,
                 0, t->text());
        break;
    }
    case CE_PushButton:{
        const QPushButton *btn = (const QPushButton*)widget;
        const_cast<LiquidStyle*>(this)->widgetAutoMasked = widget->autoMask();
        if(widget == highlightWidget)
            flags |= Style_MouseOver;
        if(btn->isDefault())
            drawPrimitive(PE_ButtonDefault, p, r, cg, flags);
        else
            drawPrimitive(PE_ButtonCommand, p, r, cg, flags);
        break;
    }
    case CE_PushButtonLabel:{
        const QPushButton *btn = (const QPushButton*)widget;
        int x, y, w, h;
        r.rect(&x, &y, &w, &h);
        bool highlighted = btn->isOn() || btn->isDown() || btn->hasFocus();

        if(btn->isOn() || btn->isDown()){
            ++x;
            ++y;
            flags |= Style_Sunken;
        }
        if(btn->isDefault()){
            int tmpX = x+4;
            int tmpY = y+(h-11)/2;
            p->setPen(highlighted ? cg.background().dark(140) :
                      cg.button().dark(140));
            p->drawLine(tmpX, tmpY, tmpX, tmpY+11);
            p->drawLine(tmpX, tmpY+11, tmpX+5, tmpY+6);
            p->setPen(highlighted ? cg.background().dark(125) :
                      cg.button().dark(125));
            p->drawLine(tmpX, tmpY, tmpX+5, tmpY+5);
            x += 8;
            w -= 8;
        }
        if(btn->isMenuButton()){
            int iw = pixelMetric(PM_MenuButtonIndicator, widget);
            QRect newRect(r.right()-iw, r.y()+2, iw-4, r.height()-4);
            drawPrimitive(PE_ArrowDown, p, newRect, cg, flags, opt);
            w -= iw;
        }
        if(btn->iconSet() && btn->iconSet()->isNull()){
            QIconSet::Mode mode = btn->isEnabled() ? QIconSet::Normal :
                QIconSet::Disabled;
            if(mode == QIconSet::Normal && btn->hasFocus())
                mode = QIconSet::Active;

            QIconSet::State state = QIconSet::Off;
            if(btn->isToggleButton() && btn->isOn())
                state = QIconSet::On;

            QPixmap pix(btn->iconSet()->pixmap(QIconSet::Small, mode, state));
            int pixW = pix.width();
            int pixH = pix.height();
            p->drawPixmap(x+4, y+h/2-pixH/2, pix);
            x += pixW+4;
            w -= pixW+4;
        }

        QColor tmp;
        if(highlighted){
            tmp = cg.background().dark(130);
            drawItem(p, QRect(x+1, y+1, w, h), AlignCenter | ShowPrefix,
                     cg, btn->isEnabled(), btn->pixmap(), btn->text(), -1,
                     &tmp);
            drawItem(p, QRect(x+2, y+2, w, h), AlignCenter | ShowPrefix,
                     cg, btn->isEnabled(), btn->pixmap(), btn->text(), -1,
                     &tmp);
        }
        tmp = btn->colorGroup().buttonText();
        drawItem(p, QRect(x, y, w, h), AlignCenter | ShowPrefix,
                 cg, btn->isEnabled(), btn->pixmap(), btn->text(), -1,
                 &tmp);
        break;
    }
    case CE_MenuBarItem:{
        QMenuItem *mi = opt.menuItem();
        bool active  = flags & Style_Active;
        bool focused = flags & Style_HasFocus;

        if(active && focused){
            drawClearBevel(p, r.x(), r.y(), r.width(), r.height(),
                           qApp->palette().active().button(), cg.background());
        }
        QColor tmpColor(cg.background().dark(120));
        drawItem(p, QRect(r.x()+1, r.y()+1, r.width(), r.height()), AlignCenter | AlignVCenter | ShowPrefix
                 | DontClip | SingleLine, cg, flags & Style_Enabled,
                 mi->pixmap(), mi->text(), -1, &tmpColor);
        drawItem(p, r, AlignCenter | AlignVCenter | ShowPrefix
                 | DontClip | SingleLine, cg, flags & Style_Enabled,
                 mi->pixmap(), mi->text());
        break;
    }
    case CE_PopupMenuItem:{
        int x, y, w, h;
        r.rect( &x, &y, &w, &h );

        const QPopupMenu *popupmenu = (const QPopupMenu *) widget;
        QMenuItem *mi = opt.menuItem();

        int tab = opt.tabWidth();
        int maxpmw = opt.maxIconWidth();
        bool dis = mi && !mi->isEnabled();
        bool checkable = popupmenu->isCheckable();
        bool act = flags & Style_Active;
        int checkcol = maxpmw;
        QColor menuBgColor;

        if(mnuHandler->transType() == None ||
           mnuHandler->transType() == StippledBg ||
           mnuHandler->transType() == TransStippleBg){
            menuBgColor = cg.background();
        }
        else if(mnuHandler->transType() == StippledBtn ||
                mnuHandler->transType() == TransStippleBtn){
            menuBgColor = cg.button();
        }
        else{
            menuBgColor = mnuHandler->bgColor();
        }

        if(checkable)
            checkcol = QMAX(maxpmw, 20);

        if(mi && mi->isSeparator()){
            p->setPen(menuBgColor.dark(130));
            p->drawLine(x, y, x+w, y);
            p->setPen(menuBgColor.light(120));
            p->drawLine(x, y+1, x+w, y+1);
            return;
        }

        if(act && mi)
            drawClearBevel(p, x, y, w, h, cg.button(), cg.background());
        else if(mnuHandler->transType() == None){
            p->fillRect(x, y, w, h, cg.background());
        }
        else if(mnuHandler->transType() == StippledBg){
            p->fillRect(x, y, w, h, bgBrush);
        }
        else if(mnuHandler->transType() == StippledBtn){
            p->fillRect(x, y, w, h, menuBrush);
        }
        else{
            KPixmap *pix = mnuHandler->pixmap(widget->winId());
            if(pix)
                p->drawPixmap(x, y, *pix, x, y, w, h);
        }

        if(!mi)
            return;

        // Menu contents drawing code based on Qt's styles. Qt is property of
        // TrollTech: www.trolltech.com. Used with permission.
        int xpos = x;
        QRect vrect = visualRect(QRect(xpos, y, checkcol, h), r);
        int xvis = vrect.x();

        // checkbox
        if(mi->isChecked()){
            if(act && !dis)
                qDrawShadePanel(p, xvis, y, checkcol, h,
                                cg, TRUE, 1, &cg.brush(QColorGroup::Button));
            else{
                qDrawShadePanel(p, xvis, y, checkcol, h, cg, TRUE, 1);
            }
        }

        // icon
        if(mi->iconSet()){
            QIconSet::Mode mode = dis ? QIconSet::Disabled : QIconSet::Normal;
            if(act && !dis)
                mode = QIconSet::Active;
            QPixmap pixmap;
            if(checkable && mi->isChecked())
                pixmap = mi->iconSet()->pixmap(QIconSet::Small, mode,
                                               QIconSet::On);
            else
                pixmap = mi->iconSet()->pixmap(QIconSet::Small, mode);
            int pixw = pixmap.width();
            int pixh = pixmap.height();
            QRect pmr(0, 0, pixw, pixh);
            pmr.moveCenter(vrect.center());
            p->setPen(cg.text());
            p->drawPixmap(pmr.topLeft(), pixmap);
        }
        else if(checkable){
            if(mi->isChecked()){
                int xp = xpos + windowsItemFrame;

                SFlags cflags = Style_Default;
                if(!dis)
                    cflags |= Style_Enabled;
                if(act)
                    cflags |= Style_On;
                drawPrimitive(PE_CheckMark, p,
                              visualRect(QRect(xp, y + windowsItemFrame,
                                               checkcol - 2*windowsItemFrame,
                                               h - 2*windowsItemFrame), r),
                              cg, cflags);
            }
        }
        QColor discol;
        if(dis){
            discol = cg.text();
            p->setPen( discol );
        }

        int xm = windowsItemFrame + checkcol + windowsItemHMargin;
        xpos += xm;

        vrect = visualRect(QRect(xpos, y+windowsItemVMargin,
                                 w-xm-tab+1, h-2*windowsItemVMargin), r);
        xvis = vrect.x();
        if(mi->custom()){
            p->save();
            if(dis && !act) {
                p->setPen(cg.light());
                mi->custom()->paint(p, cg, act, !dis,
                                    xvis+1, y+windowsItemVMargin+1,
                                    w-xm-tab+1, h-2*windowsItemVMargin);
                p->setPen(discol);
            }
            mi->custom()->paint(p, cg, act, !dis, xvis, y+windowsItemVMargin,
                                w-xm-tab+1, h-2*windowsItemVMargin);
            p->restore();
        }

        // Text
        QString s = mi->text();
        if(!s.isNull()){
            QColor textColor, shadowColor;
            if(mnuHandler->transType() == Custom)
                shadowColor = mnuHandler->bgColor().dark(130);
            else if(mnuHandler->transType() == StippledBtn ||
                    mnuHandler->transType() == TransStippleBtn)
                shadowColor = cg.button().dark(130);
            else
                shadowColor = cg.background().dark(130);

            if(dis)
                textColor = shadowColor.light(115);
            else if(mnuHandler->transType() == Custom)
                textColor = mnuHandler->textColor();
            else
                textColor = cg.text();

            int t = s.find( '\t' );
            int text_flags = AlignVCenter|ShowPrefix | DontClip | SingleLine;
            text_flags |= (QApplication::reverseLayout() ? AlignRight :
                           AlignLeft);
            if(t >= 0){                         // draw tab text
                int xp;
                xp = x + w - tab - 20 - windowsItemHMargin -
                    windowsItemFrame + 1;
                if(mnuHandler->useShadowText()){
                    p->setPen(shadowColor);
                    p->drawText(xvis+1, y+windowsItemVMargin+1, w-xm-tab+1,
                                h-2*windowsItemVMargin, text_flags, s, t);
                }
                p->setPen(textColor);
                p->drawText(xvis, y+windowsItemVMargin, w-xm-tab+1,
                            h-2*windowsItemVMargin, text_flags, s, t);
                s = s.left(t);
            }
            if(mnuHandler->useShadowText()){
                p->setPen(shadowColor);
                p->drawText(xvis+1, y+windowsItemVMargin+1, w-xm-tab+1,
                            h-2*windowsItemVMargin, text_flags, s, t);
            }
            p->setPen(textColor);
            p->drawText(xvis, y+windowsItemVMargin, w-xm-tab+1,
                        h-2*windowsItemVMargin, text_flags, s, t);
        }
        else if(mi->pixmap()){                        // draw pixmap
            QPixmap *pixmap = mi->pixmap();
            if(pixmap->depth() == 1)
                p->setBackgroundMode(OpaqueMode);
            p->drawPixmap(xvis, y+windowsItemFrame, *pixmap);
            if(pixmap->depth() == 1)
                p->setBackgroundMode(TransparentMode);
        }

        // Child menu
        if(mi->popup()){
            int dim = (h-2*windowsItemFrame) / 2;
            PrimitiveElement arrow;
            arrow = (QApplication::reverseLayout() ? PE_ArrowLeft :
                     PE_ArrowRight);
            xpos = x+w - windowsArrowHMargin - windowsItemFrame - dim;
            vrect = visualRect(QRect(xpos, y + h / 2 - dim / 2, dim, dim), r);
            if(act){
                if(!dis)
                    discol = white;
                QColorGroup g2(discol, cg.highlight(),
                               white, white,
                               dis ? discol : white,
                               discol, white);

                drawPrimitive(arrow, p, vrect,
                              g2, Style_Enabled);
            } else {
                drawPrimitive(arrow, p, vrect,
                              cg, mi->isEnabled() ? Style_Enabled :
                              Style_Default);
            }
        }
        break;
    }
    default:
        KStyle::drawControl(element, p, widget, r, cg, flags, opt);
    }
}


void LiquidStyle::drawControlMask(ControlElement element, QPainter *p,
                                  const QWidget *widget, const QRect &r,
                                  const QStyleOption& opt) const
{
    switch(element){
    case CE_PushButton:{
        if(mnuHandler->useRectangularHTML()){
            p->fillRect(r, Qt::color1);
            p->setPen(Qt::color0);
            p->drawPoint(r.x(), r.y());
            p->drawPoint(r.right(), r.y());
            p->drawPoint(r.x(), r.bottom());
            p->drawPoint(r.right(), r.bottom());
            return;
        }
        if(!htmlBtnMaskBmp){
            const_cast<LiquidStyle*>(this)->htmlBtnMaskBmp =
                new QBitmap(37, 26, htmlmask_bits, true);
            const_cast<LiquidStyle*>(this)->htmlBtnMaskBmp->
                setMask(*htmlBtnMaskBmp);
        }
        p->setPen(Qt::color1);
        int x, y, w, h;
        r.rect(&x, &y, &w, &h);
        int x2 = r.right();
        int y2 = r.bottom();
        if(w < 21 || h < 21){
            p->drawLine(x, y+2, x, y2-2); // l
            p->drawLine(x2, y+2, x2, y2-2); // r
            p->drawLine(x+2, y, x2-2, y); // t
            p->drawLine(x+2, y2, x2-2, y2); // b
            p->drawLine(x+1, y+1, x2-1, y+1);
            p->drawLine(x+1, y2-1, x2-1, y2-1);
            p->fillRect(x+1, y+2, w-2, h-4, Qt::color1);
        }
        else{
            int bx2 = htmlBtnMaskBmp->width()-1;
            int by2 = htmlBtnMaskBmp->height()-1;
            p->drawPixmap(x, y, *htmlBtnMaskBmp, 0, 0, 10, 10); // tl
            p->drawPixmap(x2-9, y, *htmlBtnMaskBmp, bx2-9, 0, 10, 10); // tr
            p->drawPixmap(x, y2-9, *htmlBtnMaskBmp, 0, by2-9, 10, 10); // bl
            p->drawPixmap(x2-9, y2-9, *htmlBtnMaskBmp, bx2-9, by2-9, 10, 10); // br
            // fills
            p->fillRect(10, y, w-19, 10, Qt::color1); // top
            p->fillRect(10, y2-9, w-19, 10, Qt::color1); // bottom
            p->fillRect(x, y+10, w+1, h-19, Qt::color1); // middle

        }
        break;
    }
    default:
        KStyle::drawControlMask(element, p, widget, r, opt);
        break;
    }
}

void LiquidStyle::drawComplexControlMask(ComplexControl control, QPainter *p,
                                         const QWidget *widget, const QRect &r,
                                         const QStyleOption &opt ) const
{
    switch (control){
    case CC_ComboBox:{
        drawControlMask(CE_PushButton, p, widget, r, opt);
        break;
    }
    default:
        KStyle::drawComplexControlMask(control, p, widget, r, opt);
    }
}

void LiquidStyle::drawComplexControl(ComplexControl control, QPainter *p,
                                     const QWidget *widget, const QRect &r,
                                     const QColorGroup &cg, SFlags flags,
                                     SCFlags controls, SCFlags active,
                                     const QStyleOption &opt) const
{
    switch(control){
    case CC_ComboBox:{
        if(controls & SC_ComboBoxArrow){
            if(widget->autoMask() && mnuHandler->useRectangularHTML()){
                drawRectangularButton(p, cg.button(), r.x(), r.y(),
                                      r.width(), r.height(),
                                      active == SC_ComboBoxArrow,
                                      widget == highlightWidget, true);
                return;
            }
            QPixmap tmpPix(r.width(), r.height());
            QPen oldPen(p->pen());
            bool sunken = (active == SC_ComboBoxArrow);
            bool hover = widget == highlightWidget;

            QPainter painter(&tmpPix);
            drawRoundButton(&painter, widget->autoMask() ?
                            qApp->palette().active().button() : cg.button(),
                            cg.background(), 0, 0,
                            r.width(), r.height(), false,
                            sunken, false, widget->autoMask());
            if(!hover){
                painter.setClipRect(0, 0, r.width()-17, r.height());
                drawRoundButton(&painter, cg.background(), cg.background(),
                                0, 0, r.width(), r.height(), false,
                                sunken, false, widget->autoMask());
            }
            painter.end();
            QPixmap *pix = btnDict.find(cg.background().rgb());
            if(!pix){
                pix = adjustHSV(*btnBorderImg, cg.background().rgb(), true);
                const_cast<LiquidStyle*>(this)->
                    btnDict.insert(cg.background().rgb(), pix);
            }
            int x2 = r.x()+r.width()-1;
            int y2 = r.y()+r.height()-1;
            int bx2 = pix->width()-1;
            int by2 = pix->height()-1;
            QBitmap btnMask(r.width(), r.height());
            const QBitmap *mask = pix->mask();
            painter.begin(&btnMask);
            painter.fillRect(0, 0, r.width(), r.height(), Qt::color0);
            painter.drawPixmap(0, 0, *mask, 0, 0, 10, 10); // tl
            painter.drawPixmap(x2-9, 0, *mask, bx2-9, 0, 10, 10); // tr
            painter.drawPixmap(0, y2-9, *mask, 0, by2-9, 10, 10); // bl
            painter.drawPixmap(x2-9, y2-9, *mask, bx2-9, by2-9, 10, 10); // br
            // fills
            painter.fillRect(10, 0, r.width()-20, 10, Qt::color1); // top
            painter.fillRect(10, y2-9, r.width()-20, 10, Qt::color1); // bottom
            painter.fillRect(0, 10, r.width(), r.height()-20, Qt::color1); // middle
            painter.end();
            tmpPix.setMask(btnMask);
            p->drawPixmap(r.x(), r.y(), tmpPix);
            //p->setPen(cg.button().dark(120));
            //p->drawLine(x2-16, r.y()+1, x2-16, y2-1);
            //p->setPen(oldPen);
            int ah = r.height() / 3;
            int aw = ah;
            int ax = r.width() - aw - 6;
            int ay = (r.height() - ah) / 2;
            // Are we enabled?
            if (widget->isEnabled())
                flags |= Style_Enabled;

            // Are we "pushed" ?
            if (active & Style_Sunken)
                flags |= Style_Sunken;
            drawPrimitive(PE_ArrowDown, p, QRect(ax, ay, aw, ah), cg, flags);
        }
        if(controls & SC_ComboBoxEditField){
            if(((const QComboBox *)widget)->editable()){
                QPen oldPen(p->pen());
                p->setPen(cg.mid());
                p->drawRect(r.x()+8, r.y()+2, r.width()-25, r.height()-4);
                p->setPen(oldPen);
            }
        }
        break;
    }
    case CC_ToolButton:{
        const QToolButton *toolbutton = (const QToolButton *) widget;

        QRect button, menuarea;
        button   = querySubControlMetrics(control, widget, SC_ToolButton, opt);
        menuarea = querySubControlMetrics(control, widget, SC_ToolButtonMenu, opt);

        SFlags bflags = flags,
            mflags = flags;

        if(active & SC_ToolButton)
            bflags |= Style_Down;
        if(active & SC_ToolButtonMenu)
            mflags |= Style_Down;

        if(widget == highlightWidget)
            bflags |= Style_MouseOver;

        if(controls & SC_ToolButton){
            // drawPrimitive(PE_ButtonTool, p, button, cg, bflags, opt);
        }
        // Draw a toolbutton menu indicator if required
        if(controls & SC_ToolButtonMenu){
            if(mflags & (Style_Down | Style_On | Style_Raised))
                drawPrimitive(PE_ButtonDropDown, p, menuarea, cg, mflags, opt);
            drawPrimitive(PE_ArrowDown, p, menuarea, cg, mflags, opt);
        }

        if(toolbutton->hasFocus() && !toolbutton->focusProxy()){
            QRect fr = toolbutton->rect();
            fr.addCoords(3, 3, -3, -3);
            drawPrimitive(PE_FocusRect, p, fr, cg);
        }
        break;
    }
    case CC_ScrollBar:{
        const_cast<LiquidStyle*>(this)->currentScrollBar =
            (QScrollBar *)widget;
        KStyle::drawComplexControl(control, p, widget, r, cg, flags,
                                   controls, active, opt);
        break;
    }
    case CC_Slider:{
        // KDE3's slider double buffering is screwy. It doesn't handle
        // color scheme changes and uses the wrong background origin
        QRect groove = querySubControlMetrics(CC_Slider, widget,
                                              SC_SliderGroove, opt);
        QRect handle = querySubControlMetrics(CC_Slider, widget,
                                              SC_SliderHandle, opt);
        if((controls & SC_SliderGroove) && groove.isValid())
            drawKStylePrimitive(KPE_SliderGroove, p, widget, groove, cg,
                                flags, opt);
        if(controls & SC_SliderTickmarks)
            QCommonStyle::drawComplexControl(control, p, widget, r, cg,
                                             flags, SC_SliderTickmarks,
                                             active, opt);
        if((controls & SC_SliderHandle) && handle.isValid())
            drawKStylePrimitive(KPE_SliderHandle, p, widget, handle, cg,
                                flags, opt);
        break;
    }
    default:
        KStyle::drawComplexControl(control, p, widget,
                                   r, cg, flags, controls, active, opt);
        break;
    }
}

QRect LiquidStyle::subRect(SubRect r, const QWidget *widget) const
{
    return(KStyle::subRect(r, widget));
}

int LiquidStyle::pixelMetric(PixelMetric m, const QWidget *widget) const
{
    switch(m){
    case PM_ButtonMargin:
        return(5);
        break;
    case PM_ButtonDefaultIndicator:
        return(0);
        break;
    case PM_ExclusiveIndicatorWidth:
    case PM_ExclusiveIndicatorHeight:
        return(16);
        break;
    case PM_IndicatorWidth:
        return(20);
        break;
    case PM_IndicatorHeight:
        return(22);
        break;
    case PM_ScrollBarExtent:
        return(15);
        break;
    case PM_ScrollBarSliderMin:
        return(16);
        break;
    case PM_SliderControlThickness:
    case PM_SliderThickness:
        return(11);
        break;
    case PM_SliderLength:
        return(10);
        break;
    case PM_TabBarTabOverlap:
    case PM_TabBarBaseOverlap:
        return(1);
        break;
    case PM_TabBarTabHSpace:
        return(18);
        break;
    case PM_TabBarTabVSpace:
        return(8);
        break;
    case PM_TabBarBaseHeight:
        return(5);
        break;
    default:
        break;
    }
    return KStyle::pixelMetric(m, widget);
}

QSize LiquidStyle::sizeFromContents(ContentsType contents,
                                    const QWidget *widget,
                                    const QSize &contentSize,
                                    const QStyleOption& opt ) const
{
    switch (contents){
    case CT_PushButton:{
        // this is a little funky - we give values not based on pixelMetric
        // because we want a custom width/height and we can only give one
        // value in pixelMetric (used in sizeHint). Odd but works well
        int w = contentSize.width()+26;
        int h = contentSize.height()+6 > 21 ? contentSize.height()+6 : 21;
        return(QSize(w, h));
    }
    case CT_PopupMenuItem:{
        if(! widget || opt.isDefault())
            break;

        const QPopupMenu *popup = (const QPopupMenu *) widget;
        bool checkable = popup->isCheckable();
        QMenuItem *mi = opt.menuItem();
        int maxpmw = opt.maxIconWidth();
        int w = contentSize.width();
        int h = contentSize.height();

        if(mi->custom()){
            w = mi->custom()->sizeHint().width();
            h = mi->custom()->sizeHint().height();
            if(!mi->custom()->fullSpan())
                h += 2*windowsItemVMargin + 2*windowsItemFrame;
        }
        else if(mi->widget()){
        }
        else if(mi->isSeparator()){
            w = 10;
            h = windowsSepHeight;
        }
        else{
            if(mi->pixmap())
                h = QMAX(h, mi->pixmap()->height() + 2*windowsItemFrame);
            else if(! mi->text().isNull())
                h = QMAX(h, popup->fontMetrics().height() + 2*windowsItemVMargin +
                         2*windowsItemFrame);
            if(mi->iconSet() != 0)
                h = QMAX(h, mi->iconSet()->pixmap(QIconSet::Small,
                                                  QIconSet::Normal).height() +
                         2*windowsItemFrame);
        }
        if(!mi->text().isNull() && mi->text().find('\t') >= 0)
            w += windowsTabSpacing;
        else if(mi->popup())
            w += 2*windowsArrowHMargin;
        if(checkable && maxpmw < 20)
            w += 20 - maxpmw;
        if(maxpmw)
            w += maxpmw + 6;
        if(checkable || maxpmw > 0)
            w += windowsCheckMarkHMargin;
        w += 20;
        return(QSize(w, h));
    }
    default:
        break;
    }
    return KStyle::sizeFromContents( contents, widget, contentSize, opt );
}


QPixmap LiquidStyle::stylePixmap(StylePixmap stylepixmap,
                                 const QWidget* widget,
                                 const QStyleOption& opt) const
{
    return KStyle::stylePixmap(stylepixmap, widget, opt);
}

bool LiquidStyle::eventFilter(QObject *obj, QEvent *ev)
{
    if(KStyle::eventFilter(obj, ev))
        return true;

#ifdef LIQUID_THREADED
    if(pulseThread)
        return(pulseEventFilter(obj, ev));
#endif
    if(obj->inherits("QPushButton") || obj->inherits("QComboBox")
            || obj->inherits("QSplitter") || obj->inherits("AppletHandleDrag")){
        QWidget *btn = (QWidget *)obj;
        if(ev->type() == QEvent::Enter){
            if(btn->isEnabled()){
                highlightWidget = btn;
                btn->repaint(false);
            }
        }
        else if(ev->type() == QEvent::Leave){
            if(btn == highlightWidget){
                highlightWidget = NULL;
                btn->repaint(false);
            }
        }
    }
    else if(obj->inherits("KToolBarButton")){
        QToolButton *btn = (QToolButton *)obj;
        if(ev->type() == QEvent::Enter){
            if(btn->isEnabled()){
                highlightWidget = btn;
                btn->repaint(false);
            }
        }
        else if(ev->type() == QEvent::Leave){
            QWidget *btn = (QWidget *)obj;
            if(btn == highlightWidget){
                highlightWidget = NULL;
                btn->repaint(false);
            }
        }
    }
    else if(obj->inherits("QScrollBar")){
        QScrollBar *sb = (QScrollBar *)obj;
        if(ev->type() == QEvent::Enter){
            if(sb->isEnabled()){
                highlightWidget = sb;
                sb->repaint(false);
            }
        }
        else if(ev->type() == QEvent::Leave){
            if(sb == highlightWidget && !sb->draggingSlider()){
                highlightWidget = NULL;
                sb->repaint(false);
            }
        }
        else if(ev->type() == QEvent::MouseButtonRelease){
            QMouseEvent *me = (QMouseEvent *)ev;
            if(sb == highlightWidget && !sb->rect().contains(me->pos())){
                highlightWidget = NULL;
                sb->repaint(false);
            }
        }
    }
    else if(obj->inherits("TaskContainer")){
        QButton *btn = (QButton *)obj;
        QPalette pal = qApp->palette();
        if(ev->type() == QEvent::Enter){
            pal.setColor(QColorGroup::Background, btnHoverColor);
            btn->setPalette(pal);
        }
        else if(ev->type() == QEvent::Leave){
            pal.setColor(QColorGroup::Background, bgColor);
            btn->setPalette(pal);
        }
    }
    else if(obj->inherits("KMiniPagerButton")){
        QButton *btn = (QButton *)obj;
        if(ev->type() == QEvent::Paint){
            if(!(btn->isOn() || btn->isDown())){
                QPalette pal = btn->palette();
                pal.setBrush(QColorGroup::Dark, btn == highlightWidget ?
                             pagerHoverBrush : pagerBrush);
                btn->setPalette(pal);
            }
            else{
                QPalette pal = btn->palette();
                pal.setBrush(QColorGroup::Dark,
                             QApplication::palette().active().brush(QColorGroup::Dark));
                btn->setPalette(pal);

            }
        }
        else if(ev->type() == QEvent::Enter){
            highlightWidget = btn;
            btn->repaint(false);
        }
        else if(ev->type() == QEvent::Leave){
            highlightWidget = NULL;
            btn->repaint(false);
        }

    }
    else if(obj->inherits("QLineEdit")){
        if(obj->parent() && obj->parent()->inherits("QComboBox")){
            QWidget *btn = (QWidget *)obj->parent();
            if(ev->type() == QEvent::Enter){
                if (btn->isEnabled()){
                    highlightWidget = btn;
                    btn->repaint(false);
                }
            }
            else if(ev->type() == QEvent::Leave){
                if (btn == highlightWidget)
                    highlightWidget = NULL;
                btn->repaint(false);
            }
        }
    }
    else if(obj->inherits("QRadioButton") || obj->inherits("QCheckBox")){
        QButton *btn = (QButton *)obj;
        bool isRadio = obj->inherits("QRadioButton");
        if(ev->type() == QEvent::Paint){
            //if(btn->autoMask())
            btn->erase();
            QPainter p;
            p.begin(btn);
            QFontMetrics fm = btn->fontMetrics();
            QSize lsz = fm.size(ShowPrefix, btn->text());
            QSize sz = isRadio ?
                QSize(pixelMetric(PM_ExclusiveIndicatorWidth),
                      pixelMetric(PM_ExclusiveIndicatorHeight)) :
                QSize(pixelMetric(PM_IndicatorWidth),
                      pixelMetric(PM_IndicatorHeight));
            int x = 0;
            int y = (btn->height()-lsz.height()+fm.height()-sz.height())/2;
            SFlags flags = Style_Default;
            if (btn->isEnabled())
                flags |= Style_Enabled;
            if (btn->hasFocus())
                flags |= Style_HasFocus;
            if (btn->isDown())
                flags |= Style_Down;
            if (highlightWidget == btn)
                flags |= Style_MouseOver;

            if (btn->state() == QButton::On)
                flags |= Style_On;
            else if (btn->state() == QButton::Off)
                flags |= Style_Off;
            if(isRadio)
                drawControl(CE_RadioButton, &p, btn,
                            QRect(x, y, sz.width(), sz.height()),
                            btn->colorGroup(), flags);
            else
                drawControl(CE_CheckBox, &p, btn,
                            QRect(x, y, sz.width(), sz.height()),
                            btn->colorGroup(), flags);
            x = sz.width() + 6;
            y = 0;
            if(btn->hasFocus()){
                QColor tmp(btn->colorGroup().background().dark(130));
                drawItem(&p, QRect(sz.width()+6+2, 1, btn->width()-(sz.width()+6+1),
                                   btn->height()), AlignLeft|AlignVCenter|ShowPrefix,
                         btn->colorGroup(), btn->isEnabled(),
                         btn->pixmap(), btn->text(), -1, &tmp);
                drawItem(&p, QRect(sz.width()+6+3, 2, btn->width()-(sz.width()+6+1),
                                   btn->height()), AlignLeft|AlignVCenter|ShowPrefix,
                         btn->colorGroup(), btn->isEnabled(),
                         btn->pixmap(), btn->text(), -1, &tmp);

            }
            drawItem(&p, QRect(sz.width()+6+1, 0, btn->width()-(sz.width()+6+1),
                     btn->height()), AlignLeft|AlignVCenter|ShowPrefix,
                     btn->colorGroup(), btn->isEnabled(),
                     btn->pixmap(), btn->text());
            p.end();
            return(true);
        }
        // for hover, just redraw the indicator (not the text)
        else if((ev->type() == QEvent::Enter && btn->isEnabled()) ||
                (ev->type() == QEvent::Leave && btn == highlightWidget)){
            QButton *btn = (QButton *)obj;
            bool isRadio = obj->inherits("QRadioButton");

            if(ev->type() == QEvent::Enter)
                highlightWidget = btn;
            else 
                highlightWidget = NULL;
            QFontMetrics fm = btn->fontMetrics();
            QSize lsz = fm.size(ShowPrefix, btn->text());
            QSize sz = isRadio ?
                QSize(pixelMetric(PM_ExclusiveIndicatorWidth),
                      pixelMetric(PM_ExclusiveIndicatorHeight)) :
                QSize(pixelMetric(PM_IndicatorWidth),
                      pixelMetric(PM_IndicatorHeight));
            int x = 0;
            int y = (btn->height()-lsz.height()+fm.height()-sz.height())/2;
            //if(btn->autoMask())
            //    btn->erase(x+1, y+1, sz.width()-2, sz.height()-2);
            QPainter p;
            p.begin(btn);
            SFlags flags = Style_Default;
            if (btn->isEnabled())
                flags |= Style_Enabled;
            if (btn->hasFocus())
                flags |= Style_HasFocus;
            if (btn->isDown())
                flags |= Style_Down;
            if (highlightWidget == btn)
                flags |= Style_MouseOver;

            if (btn->state() == QButton::On)
                flags |= Style_On;
            else if (btn->state() == QButton::Off)
                flags |= Style_Off;
            if(isRadio)
                drawControl(CE_RadioButton, &p, btn,
                            QRect(x, y, sz.width(), sz.height()),
                            btn->colorGroup(), flags);
            else
                drawControl(CE_CheckBox, &p, btn,
                            QRect(x, y, sz.width(), sz.height()),
                            btn->colorGroup(), flags);
            p.end();
        }
    }
    else if(obj->inherits("QHeader")){
        QHeader *hw = (QHeader *)obj;
        if(ev->type() == QEvent::Enter){
            currentHeader = hw;
            headerHoverID = -1;
        }
        else if(ev->type() == QEvent::Leave){
            currentHeader = NULL;
            if(headerHoverID != -1){
                hw->repaint(hw->sectionPos(headerHoverID), 0,
                            hw->sectionSize(headerHoverID), hw->height());
            }
            headerHoverID = -1;
        }
        else if(ev->type() == QEvent::MouseMove){
            QMouseEvent *me = (QMouseEvent *)ev;
            int oldHeader = headerHoverID;
            headerHoverID = hw->sectionAt(me->x());
            if(oldHeader != headerHoverID){
                // reset old header
                if(oldHeader != -1){
                    hw->repaint(hw->sectionPos(oldHeader), 0,
                                hw->sectionSize(oldHeader), hw->height());
                }
                if(headerHoverID != -1){
                    hw->repaint(hw->sectionPos(headerHoverID), 0,
                                hw->sectionSize(headerHoverID), hw->height());
                }
            }
        }
    }
    else if(obj->inherits("QTipLabel")){
        if(ev->type() == QEvent::PaletteChange ||
           ev->type() == QEvent::ApplicationPaletteChange){
            qWarning("Got tiplabel palette change"); // Debug
            ((QWidget*)obj)->setPalette(tooltipPalette);
            return(true);
        }
    }
    return(false);
}

QRect LiquidStyle::querySubControlMetrics(ComplexControl control, const QWidget
                                          *widget, SubControl subcontrol,
                                          const QStyleOption &opt) const
{      
    if(control == CC_ComboBox && subcontrol == SC_ComboBoxEditField)
        return(QRect(9, 3, widget->width()-27, widget->height()-6));
    else
        return(KStyle::querySubControlMetrics(control, widget, subcontrol, opt));
}

void LiquidStyle::polishPopupMenu(QPopupMenu *mnu)
{
    mnu->installEventFilter(mnuHandler);
    KStyle::polishPopupMenu(mnu);
    // isn't getting caught in polish...
    mnu->setBackgroundMode(QWidget::NoBackground);
}


#include "liquid.moc"

