#ifndef __LIQUID_STYLE_H
#define __LIQUID_STYLE_H

#include <qbitmap.h>
#include <qintdict.h>
#include <qstyle.h>
#include <qimage.h>
#include <qheader.h>
#include <qpainter.h>
#include <qscrollbar.h>
#include <kpixmap.h>
#include <kstyle.h>

#ifdef LIQUID_THREADED

#include <qthread.h>
#include <qwaitcondition.h>

#endif

#define BITMAP_ITEMS 50 // Don't worry, they're demand loaded ;-)
#define LIQUID_MENU_CHANGE 667
#define MOSFET_BUTTON_CHANGE 661

class LiquidStyle;

enum BitmapData{RadioOn=0, RadioOff, RadioOnHover, RadioOffHover, VSBSliderTop,
VSBSliderMid, VSBSliderBtm, VSBSliderTopHover, VSBSliderMidHover,
VSBSliderBtmHover, VSBSliderTopBg, VSBSliderMidBg, VSBSliderBtmBg,
HSBSliderTop, HSBSliderMid, HSBSliderBtm, HSBSliderTopHover, HSBSliderMidHover,
HSBSliderBtmHover, HSBSliderTopBg, HSBSliderMidBg,
HSBSliderBtmBg, Tab, TabDown, TabFocus, ButtonShadow, CB, CBDown, CBHover,
CBDownHover, HSlider, VSlider, HTMLBtnBorder, HTMLBtnBorderDown, HTMLCB,
HTMLCBDown, HTMLCBHover, HTMLCBDownHover, HTMLRadio, HTMLRadioDown,
HTMLRadioHover, HTMLRadioDownHover,

RadioOnPulse, RadioOffPulse, HTMLRadioPulse, HTMLRadioDownPulse,
CBPulse, CBDownPulse, HTMLCBPulse, HTMLCBDownPulse};


enum PulseType{PushButton, ComboBox, AppletHandle, Splitter, ToolButton};

enum TransType{None=0, StippledBg, StippledBtn, TransStippleBg,
    TransStippleBtn, Custom};

class TransMenuHandler : public QObject
{
    Q_OBJECT
public:
    TransMenuHandler(QObject *parent);
    ~TransMenuHandler(){;}
    void reloadSettings();
    int transType(){return(type);}
    KPixmap *pixmap(WId id){return(pixDict.find(id));}
    const QColor& textColor(){return(fgColor);}
    const QColor& bgColor(){return(color);}
    bool useShadowText(){return(shadowText);}

    bool useBgStipple(){return(bgStipple);}
    bool usePanelStipple(){return(panelStipple);}
    bool useReverseBtnColor(){return(reverseBtnColor);}
    bool usePanelCustomColor(){return(panelCustom);}
    QColor panelCustomColor(){return(customPanelColor);}
    bool useRectangularHTML(){return(rectangularHTML);}
protected:
    bool eventFilter(QObject *obj, QEvent *ev);
    void stripePixmap(KPixmap &pix, const QColor &color);

    QColor color, fgColor;
    int opacity;
    int type;
    bool shadowText;

    bool bgStipple, panelStipple, reverseBtnColor, panelCustom, rectangularHTML;
    QColor customPanelColor;

    QIntDict<KPixmap>pixDict;

};

#ifdef LIQUID_THREADED
class LiquidPulseThread : public QThread
{
public:
    LiquidPulseThread(LiquidStyle *s);
    QColor* color(){return(&c);}
    int frameCount(){return(count);}
    void clearCache(){btnPulseDict.clear(); bevelPulseDict.clear();}
protected:
    void run();

    // draws in buffer
    void drawRoundPulseFrame(const QColor &c, const QColor &bg,
                             int x, int y, int w, int h,
                             bool supportPushDown);
    void drawBevelPulseFrame(int x, int y, int w, int h,
                             const QColor &c, const QColor &bg);
    LiquidStyle *style;
    int count;
    QColor c;
    QIntDict<QPixmap>btnPulseDict;
    QIntDict<QPixmap>bevelPulseDict;
    QPixmap *buffer;
    QPainter *p;
};
#endif

class LiquidStyle : public KStyle
{
    Q_OBJECT
    friend class LiquidPulseThread;
public:
    LiquidStyle();
    virtual ~LiquidStyle();

    virtual void polish(QWidget *w);
    virtual void unPolish(QWidget *w);
    //virtual void polish(QApplication *a);
    virtual void unPolish(QApplication *a);
    virtual void polish(QPalette &p);
    void polishPopupMenu(QPopupMenu *p);

    void drawKStylePrimitive(KStylePrimitive kpe, QPainter* p,
                             const QWidget* widget, const QRect &r,
                             const QColorGroup &cg,
                             SFlags flags = Style_Default,
                             const QStyleOption& = QStyleOption::Default) const;
    void drawPrimitive(PrimitiveElement pe, QPainter* p, const QRect &r,
                       const QColorGroup &cg, SFlags flags = Style_Default,
                       const QStyleOption& = QStyleOption::Default) const;
    void drawControl(ControlElement element, QPainter *p,
                     const QWidget *widget, const QRect &r,
                     const QColorGroup &cg, SFlags flags = Style_Default,
                     const QStyleOption& = QStyleOption::Default) const;
    void drawControlMask(ControlElement element, QPainter *p,
                         const QWidget *widget, const QRect &r,
                         const QStyleOption& = QStyleOption::Default) const;
    void drawComplexControl(ComplexControl control, QPainter *p,
                            const QWidget *widget, const QRect &r,
                            const QColorGroup &cg,
                            SFlags flags = Style_Default,
                            SCFlags controls = SC_All,
                            SCFlags active = SC_None,
                            const QStyleOption& = QStyleOption::Default) const;
    void drawComplexControlMask(ComplexControl control, QPainter *p,
                                const QWidget *widget, const QRect &r,
                                const QStyleOption& = QStyleOption::Default) const;

    int pixelMetric(PixelMetric m, const QWidget *widget=0) const;

    QSize sizeFromContents(ContentsType contents, const QWidget *widget,
                           const QSize &contentSize, const QStyleOption& opt) const;
    QRect subRect(SubRect r, const QWidget *widget ) const;
    QRect querySubControlMetrics(ComplexControl control, const QWidget *widget,
                                 SubControl subcontrol, const QStyleOption &opt = QStyleOption::Default) const;
    // Fix Qt3's wacky image positions
    QPixmap stylePixmap(StylePixmap stylepixmap, const QWidget *widget=0,
                        const QStyleOption& = QStyleOption::Default ) const;
    bool eventFilter(QObject *object, QEvent *event);
#ifdef LIQUID_THREADED
    bool pulseEventFilter(QObject *object, QEvent *event);
#endif
protected:
    QPixmap* getPixmap(BitmapData item) const;
    QPixmap* adjustHSV(QImage &img, const QColor &c, bool blend=false) const;
    QImage* adjustHSVImage(QImage &img, const QColor &c, bool blend=false) const;
    void adjustHSV(QPixmap &pix, const QColor &c) const;
    void adjustIntensity(QPixmap *pix, int val);
    QPixmap* processEmbedded(const char *label, const QColor &c,
                             bool blend = false) const;
    void drawRoundButton(QPainter *p, const QColor &c, const QColor &bg, int x,
                         int y, int w, int h, bool supportPushDown = false,
                         bool pushedDown = false, bool autoDefault = false,
                         bool isMasked = false) const;
    void drawClearBevel(QPainter *p, int x, int y, int w, int h,
                        const QColor &c, const QColor &bg) const;
    void drawRectangularButton(QPainter *p, const QColor &c,
                               int x, int y, int w, int h,
                               bool sunken=false, bool hover=false,
                               bool isCombo=false) const;
    //QColor getColorEntry(const QString &value);
    QColor calcBtnHoverColor(const QColor &c, const QColor &bg);
private:
    LiquidStyle( const LiquidStyle & );
    LiquidStyle& operator=( const LiquidStyle & );

    QBrush wallpaper;
    QBitmap *sbLeft, *sbRight, *sbUp, *sbDown;
    QBitmap *htmlBtnMaskBmp, *htmlBtnFrameBmp;
    QImage *btnBorderImg;
    QPixmap *btnBlendPix, *bevelFillPix, *smallBevelFillPix, *menuPix;
    QBrush baseBrush, menuBrush, pagerBrush, pagerHoverBrush, bgBrush;

    QPixmap *vsbSliderFillPix;
    QPixmap *pixmaps[BITMAP_ITEMS];
    QPixmap sbBuffer;
    QScrollBar *currentScrollBar;
    QWMatrix rMatrix;
    int lowLightVal;
    QColor btnColor, btnHoverColor, bgColor, bgHoverColor;

    QHeader *currentHeader;
    int headerHoverID;

    bool widgetAutoMasked;

    bool oldPaletteLoaded;
    QPalette oldPalette, polishedPalette, tooltipPalette;
    QDateTime qtrcModificationTime;

    TransMenuHandler *mnuHandler;

    QIntDict<QPixmap>btnDict;
    QIntDict<QPixmap>btnShadowedDict;
    QIntDict<QPixmap>bevelFillDict;
    QIntDict<QPixmap>smallBevelFillDict;

    QWidget *highlightWidget;

#ifdef LIQUID_THREADED
    bool inApp;
    bool inPulse;
    PulseType pulseWidgetType;
    LiquidPulseThread *pulseThread;
    QWaitCondition *pulseWait;
#endif
};

#endif
